@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -55,7 +55,7 @@
 #include <stdlib.h>
 #include <limits.h>
 #if defined(Q_OS_SYMBIAN)
-# include <syslimits.h>
+# include <sys/syslimits.h>
 # include <f32file.h>
 # include <pathinfo.h>
 # include "private/qcore_symbian_p.h"
@@ -81,6 +81,40 @@ static bool isRelativePathSymbian(const QString& fileName)
              && ((fileName.at(0).isLetter() && fileName.at(1) == QLatin1Char(':'))
              || (fileName.at(0) == QLatin1Char('/') && fileName.at(1) == QLatin1Char('/')))));
 }
+
+/*!
+ \internal
+ convert symbian error code to the one suitable for setError.
+ example usage: setSymbianError(err, QFile::CopyError, QLatin1String("copy error"))
+*/
+void QFSFileEnginePrivate::setSymbianError(int symbianError, QFile::FileError defaultError, QString defaultString)
+{
+    Q_Q(QFSFileEngine);
+    switch (symbianError) {
+    case KErrNone:
+        q->setError(QFile::NoError, QLatin1String(""));
+        break;
+    case KErrAccessDenied:
+        q->setError(QFile::PermissionsError, QLatin1String("access denied"));
+        break;
+    case KErrPermissionDenied:
+        q->setError(QFile::PermissionsError, QLatin1String("permission denied"));
+        break;
+    case KErrAbort:
+        q->setError(QFile::AbortError, QLatin1String("aborted"));
+        break;
+    case KErrCancel:
+        q->setError(QFile::AbortError, QLatin1String("cancelled"));
+        break;
+    case KErrTimedOut:
+        q->setError(QFile::TimeOutError, QLatin1String("timed out"));
+        break;
+    default:
+        q->setError(defaultError, defaultString);
+        break;
+    }
+}
+
 #endif
 
 /*!
@@ -88,7 +122,7 @@ static bool isRelativePathSymbian(const QString& fileName)
 
     Returns the stdlib open string corresponding to a QIODevice::OpenMode.
 */
-static inline QByteArray openModeToFopenMode(QIODevice::OpenMode flags, const QString &fileName)
+static inline QByteArray openModeToFopenMode(QIODevice::OpenMode flags, const QByteArray &fileName)
 {
     QByteArray mode;
     if ((flags & QIODevice::ReadOnly) && !(flags & QIODevice::Truncate)) {
@@ -96,9 +130,9 @@ static inline QByteArray openModeToFopenMode(QIODevice::OpenMode flags, const QS
         if (flags & QIODevice::WriteOnly) {
             QT_STATBUF statBuf;
             if (!fileName.isEmpty()
-                && QT_STAT(QFile::encodeName(fileName), &statBuf) == 0
+                && QT_STAT(fileName, &statBuf) == 0
                 && (statBuf.st_mode & S_IFMT) == S_IFREG) {
-                mode += "+";
+                mode += '+';
             } else {
                 mode = "wb+";
             }
@@ -191,12 +225,16 @@ bool QFSFileEnginePrivate::nativeOpen(QIODevice::OpenMode openMode)
             return false;
         }
 
-        QT_STATBUF statBuf;
-        if (QT_FSTAT(fd, &statBuf) != -1) {
-            if ((statBuf.st_mode & S_IFMT) == S_IFDIR) {
-                q->setError(QFile::OpenError, QLatin1String("file to open is a directory"));
-                QT_CLOSE(fd);
-                return false;
+        if (!(openMode & QIODevice::WriteOnly)) {
+            // we don't need this check if we tried to open for writing because then
+            // we had received EISDIR anyway.
+            QT_STATBUF statBuf;
+            if (QT_FSTAT(fd, &statBuf) != -1) {
+                if ((statBuf.st_mode & S_IFMT) == S_IFDIR) {
+                    q->setError(QFile::OpenError, QLatin1String("file to open is a directory"));
+                    QT_CLOSE(fd);
+                    return false;
+                }
             }
         }
 
@@ -216,7 +254,7 @@ bool QFSFileEnginePrivate::nativeOpen(QIODevice::OpenMode openMode)
 
         fh = 0;
     } else {
-        QByteArray fopenMode = openModeToFopenMode(openMode, filePath);
+        QByteArray fopenMode = openModeToFopenMode(openMode, nativeFilePath.constData());
 
         // Try to open the file in buffered mode.
         do {
@@ -230,12 +268,16 @@ bool QFSFileEnginePrivate::nativeOpen(QIODevice::OpenMode openMode)
             return false;
         }
 
-        QT_STATBUF statBuf;
-        if (QT_FSTAT(fileno(fh), &statBuf) != -1) {
-            if ((statBuf.st_mode & S_IFMT) == S_IFDIR) {
-                q->setError(QFile::OpenError, QLatin1String("file to open is a directory"));
-                fclose(fh);
-                return false;
+        if (!(openMode & QIODevice::WriteOnly)) {
+            // we don't need this check if we tried to open for writing because then
+            // we had received EISDIR anyway.
+            QT_STATBUF statBuf;
+            if (QT_FSTAT(fileno(fh), &statBuf) != -1) {
+                if ((statBuf.st_mode & S_IFMT) == S_IFDIR) {
+                    q->setError(QFile::OpenError, QLatin1String("file to open is a directory"));
+                    fclose(fh);
+                    return false;
+                }
             }
         }
 
@@ -419,8 +461,10 @@ bool QFSFileEngine::copy(const QString &newName)
         }
     ) // End TRAP
     delete fm;
-    // ### Add error reporting on failure
-    return (err == KErrNone);
+    if (err == KErrNone)
+        return true;
+    d->setSymbianError(err, QFile::CopyError, QLatin1String("copy error"));
+    return false;
 #else
     Q_UNUSED(newName);
     // ### Add copy code for Unix here
@@ -590,13 +634,8 @@ QString QFSFileEngine::homePath()
 QString QFSFileEngine::rootPath()
 {
 #if defined(Q_OS_SYMBIAN)
-# ifdef Q_WS_S60
     TFileName symbianPath = PathInfo::PhoneMemoryRootPath();
     return QDir::cleanPath(QDir::fromNativeSeparators(qt_TDesC2QString(symbianPath)));
-# else
-# warning No fallback implementation of QFSFileEngine::rootPath()
-    return QString();
-# endif
 #else
     return QLatin1String("/");
 #endif
@@ -605,17 +644,12 @@ QString QFSFileEngine::rootPath()
 QString QFSFileEngine::tempPath()
 {
 #if defined(Q_OS_SYMBIAN)
-# ifdef Q_WS_S60
     TFileName symbianPath = PathInfo::PhoneMemoryRootPath();
     QString temp = QDir::fromNativeSeparators(qt_TDesC2QString(symbianPath));
     temp += QLatin1String( "temp/");
 
     // Just to verify that folder really exist on hardware
     QT_MKDIR(QFile::encodeName(temp), 0777);
-# else
-# warning No fallback implementation of QFSFileEngine::tempPath()
-    QString temp;
-# endif
 #else
     QString temp = QFile::decodeName(qgetenv("TMPDIR"));
     if (temp.isEmpty())
@@ -660,6 +694,16 @@ bool QFSFileEnginePrivate::doStat() const
             could_stat = (QT_FSTAT(QT_FILENO(fh), &st) == 0);
         } else if (fd == -1) {
             // ### actually covers two cases: d->fh and when the file is not open
+#if defined(Q_OS_SYMBIAN)
+            // Optimisation for Symbian where fileFlags() calls both doStat() and isSymlink(), but rarely on real links.
+            // When the filename is not a link, lstat will return the same info as stat, but this also removes
+            // any need for a further call to lstat to check if the file is a link.
+            need_lstat = false;
+            could_stat = (QT_LSTAT(nativeFilePath.constData(), &st) == 0);
+            is_link = could_stat ? S_ISLNK(st.st_mode) : false;
+            // if it turns out this was a link, we can call stat too.
+            if (is_link)
+#endif
             could_stat = (QT_STAT(nativeFilePath.constData(), &st) == 0);
         } else {
             could_stat = (QT_FSTAT(fd, &st) == 0);
@@ -718,6 +762,46 @@ static bool _q_isMacHidden(const QString &path)
 }
 #endif
 
+QAbstractFileEngine::FileFlags QFSFileEnginePrivate::getPermissions(QAbstractFileEngine::FileFlags type) const
+{
+    QAbstractFileEngine::FileFlags ret = 0;
+
+    if (st.st_mode & S_IRUSR)
+        ret |= QAbstractFileEngine::ReadOwnerPerm;
+    if (st.st_mode & S_IWUSR)
+        ret |= QAbstractFileEngine::WriteOwnerPerm;
+    if (st.st_mode & S_IXUSR)
+        ret |= QAbstractFileEngine::ExeOwnerPerm;
+    if (st.st_mode & S_IRGRP)
+        ret |= QAbstractFileEngine::ReadGroupPerm;
+    if (st.st_mode & S_IWGRP)
+        ret |= QAbstractFileEngine::WriteGroupPerm;
+    if (st.st_mode & S_IXGRP)
+        ret |= QAbstractFileEngine::ExeGroupPerm;
+    if (st.st_mode & S_IROTH)
+        ret |= QAbstractFileEngine::ReadOtherPerm;
+    if (st.st_mode & S_IWOTH)
+        ret |= QAbstractFileEngine::WriteOtherPerm;
+    if (st.st_mode & S_IXOTH)
+        ret |= QAbstractFileEngine::ExeOtherPerm;
+
+    // calculate user permissions
+    if (type & QAbstractFileEngine::ReadUserPerm) {
+        if (QT_ACCESS(nativeFilePath.constData(), R_OK) == 0)
+            ret |= QAbstractFileEngine::ReadUserPerm;
+    }
+    if (type & QAbstractFileEngine::WriteUserPerm) {
+        if (QT_ACCESS(nativeFilePath.constData(), W_OK) == 0)
+            ret |= QAbstractFileEngine::WriteUserPerm;
+    }
+    if (type & QAbstractFileEngine::ExeUserPerm) {
+        if (QT_ACCESS(nativeFilePath.constData(), X_OK) == 0)
+            ret |= QAbstractFileEngine::ExeUserPerm;
+    }
+
+    return ret;
+}
+
 /*!
     \reimp
 */
@@ -737,32 +821,8 @@ QAbstractFileEngine::FileFlags QFSFileEngine::fileFlags(FileFlags type) const
     if (!exists && !d->isSymlink())
         return ret;
 
-    if (exists && (type & PermsMask)) {
-        if (d->st.st_mode & S_IRUSR)
-            ret |= ReadOwnerPerm;
-        if (d->st.st_mode & S_IWUSR)
-            ret |= WriteOwnerPerm;
-        if (d->st.st_mode & S_IXUSR)
-            ret |= ExeOwnerPerm;
-        if (d->st.st_mode & S_IRUSR)
-            ret |= ReadUserPerm;
-        if (d->st.st_mode & S_IWUSR)
-            ret |= WriteUserPerm;
-        if (d->st.st_mode & S_IXUSR)
-            ret |= ExeUserPerm;
-        if (d->st.st_mode & S_IRGRP)
-            ret |= ReadGroupPerm;
-        if (d->st.st_mode & S_IWGRP)
-            ret |= WriteGroupPerm;
-        if (d->st.st_mode & S_IXGRP)
-            ret |= ExeGroupPerm;
-        if (d->st.st_mode & S_IROTH)
-            ret |= ReadOtherPerm;
-        if (d->st.st_mode & S_IWOTH)
-            ret |= WriteOtherPerm;
-        if (d->st.st_mode & S_IXOTH)
-            ret |= ExeOtherPerm;
-    }
+    if (exists && (type & PermsMask))
+        ret |= d->getPermissions(type);
     if (type & TypesMask) {
 #if !defined(QWS) && defined(Q_OS_MAC)
         bool foundAlias = false;
@@ -817,10 +877,9 @@ QAbstractFileEngine::FileFlags QFSFileEngine::fileFlags(FileFlags type) const
             ret |= RootFlag;
         } else {
             QString baseName = fileName(BaseName);
-            if ((baseName.size() > 1
-                 && baseName.at(0) == QLatin1Char('.') && baseName.at(1) != QLatin1Char('.'))
+            if ((baseName.size() > 0 && baseName.at(0) == QLatin1Char('.'))
 #  if !defined(QWS) && defined(Q_OS_MAC)
-                    || _q_isMacHidden(d->filePath)
+                || _q_isMacHidden(d->filePath)
 #  endif
             ) {
                 ret |= HiddenFlag;
@@ -964,7 +1023,7 @@ QString QFSFileEngine::fileName(FileName file) const
 #if !defined(QWS) && defined(Q_OS_MAC)
         QCFType<CFURLRef> url = CFURLCreateWithFileSystemPath(0, QCFString(d->filePath),
                                                               kCFURLPOSIXPathStyle, true);
-        if (CFDictionaryRef dict = CFBundleCopyInfoDictionaryForURL(url)) {
+        if (QCFType<CFDictionaryRef> dict = CFBundleCopyInfoDictionaryForURL(url)) {
             if (CFTypeRef name = (CFTypeRef)CFDictionaryGetValue(dict, kCFBundleNameKey)) {
                 if (CFGetTypeID(name) == CFStringGetTypeID())
                     return QCFString::toQString((CFStringRef)name);
@@ -1082,7 +1141,7 @@ QString QFSFileEngine::fileName(FileName file) const
                 if (FSResolveAliasFile(&fref, true, &isFolder, &isAlias) == noErr && isAlias) {
                     AliasHandle alias;
                     if (FSNewAlias(0, &fref, &alias) == noErr && alias) {
-                        CFStringRef cfstr;
+                        QCFString cfstr;
                         if (FSCopyAliasInfo(alias, 0, 0, &cfstr, 0, 0) == noErr)
                             return QCFString::toQString(cfstr);
                     }
@@ -1263,7 +1322,7 @@ uchar *QFSFileEnginePrivate::map(qint64 offset, qint64 size, QFile::MemoryMapFla
     int pageSize = getpagesize();
     int extra = offset % pageSize;
 
-    if (size + extra > (size_t)-1) {
+    if (quint64(size + extra) > quint64((size_t)-1)) {
         q->setError(QFile::UnspecifiedError, qt_error_string(int(EINVAL)));
         return 0;
     }
