@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -43,6 +43,7 @@
 #include "qplatformdefs.h"
 #include "qabstractfileengine.h"
 #include "private/qfsfileengine_p.h"
+#include <private/qsystemlibrary_p.h>
 #include <qdebug.h>
 
 #include "qfile.h"
@@ -66,10 +67,6 @@
 #include <ctype.h>
 #include <limits.h>
 #define SECURITY_WIN32
-#ifdef Q_CC_MINGW
-// A workaround for a certain version of MinGW, the define UNICODE_STRING.
-#include <subauth.h>
-#endif
 #include <security.h>
 
 #ifndef _INTPTR_T_DEFINED
@@ -181,7 +178,7 @@ void QFSFileEnginePrivate::resolveLibs()
 
         triedResolve = true;
 #if !defined(Q_OS_WINCE)
-        HINSTANCE advapiHnd = LoadLibraryW(L"advapi32");
+        HINSTANCE advapiHnd = QSystemLibrary::load(L"advapi32");
         if (advapiHnd) {
             ptrGetNamedSecurityInfoW = (PtrGetNamedSecurityInfoW)GetProcAddress(advapiHnd, "GetNamedSecurityInfoW");
             ptrLookupAccountSidW = (PtrLookupAccountSidW)GetProcAddress(advapiHnd, "LookupAccountSidW");
@@ -213,7 +210,7 @@ void QFSFileEnginePrivate::resolveLibs()
                 ptrFreeSid(pWorld);
             }
         }
-        HINSTANCE userenvHnd = LoadLibraryW(L"userenv");
+        HINSTANCE userenvHnd = QSystemLibrary::load(L"userenv");
         if (userenvHnd)
             ptrGetUserProfileDirectoryW = (PtrGetUserProfileDirectoryW)GetProcAddress(userenvHnd, "GetUserProfileDirectoryW");
 #endif
@@ -221,7 +218,6 @@ void QFSFileEnginePrivate::resolveLibs()
 }
 #endif // QT_NO_LIBRARY
 
-// UNC functions NT
 typedef DWORD (WINAPI *PtrNetShareEnum)(LPWSTR, DWORD, LPBYTE*, DWORD, LPDWORD, LPDWORD, LPDWORD);
 static PtrNetShareEnum ptrNetShareEnum = 0;
 typedef DWORD (WINAPI *PtrNetApiBufferFree)(LPVOID);
@@ -245,7 +241,7 @@ bool QFSFileEnginePrivate::resolveUNCLibs()
 #endif
         triedResolve = true;
 #if !defined(Q_OS_WINCE)
-        HINSTANCE hLib = LoadLibraryW(L"Netapi32");
+        HINSTANCE hLib = QSystemLibrary::load(L"Netapi32");
         if (hLib) {
             ptrNetShareEnum = (PtrNetShareEnum)GetProcAddress(hLib, "NetShareEnum");
             if (ptrNetShareEnum)
@@ -455,13 +451,27 @@ bool QFSFileEnginePrivate::nativeClose()
 
     // Windows native mode.
     bool ok = true;
+
+#ifndef Q_OS_WINCE
+    if (cachedFd != -1) {
+        if (::_close(cachedFd) && !::CloseHandle(fileHandle)) {
+            q->setError(QFile::UnspecifiedError, qt_error_string());
+            ok = false;
+        }
+
+        // System handle is closed with associated file descriptor.
+        fileHandle = INVALID_HANDLE_VALUE;
+        cachedFd = -1;
+
+        return ok;
+    }
+#endif
+
     if ((fileHandle == INVALID_HANDLE_VALUE || !::CloseHandle(fileHandle))) {
         q->setError(QFile::UnspecifiedError, qt_error_string());
         ok = false;
     }
     fileHandle = INVALID_HANDLE_VALUE;
-    cachedFd = -1;              // gets closed by CloseHandle above
-
     return ok;
 }
 
@@ -533,11 +543,13 @@ qint64 QFSFileEnginePrivate::nativeSize() const
             if (errorCode == ERROR_ACCESS_DENIED || errorCode == ERROR_SHARING_VIOLATION) {
                 QByteArray path = nativeFilePath;
                 // path for the FindFirstFile should not end with a trailing slash
-                while (path.endsWith('\\'))
-                    path.chop(1);
+                while (!path.isEmpty() && reinterpret_cast<const wchar_t *>(
+                                path.constData() + path.length())[-1] == '\\')
+                    path.chop(2);
 
                 // FindFirstFile can not handle drives
-                if (!path.endsWith(':')) {
+                if (!path.isEmpty() && reinterpret_cast<const wchar_t *>(
+                                path.constData() + path.length())[-1] != ':') {
                     WIN32_FIND_DATA findData;
                     HANDLE hFind = ::FindFirstFile((const wchar_t*)path.constData(),
                                                    &findData);
@@ -1042,11 +1054,11 @@ QString QFSFileEngine::homePath()
         if (ok) {
             DWORD dwBufferSize = 0;
             // First call, to determine size of the strings (with '\0').
-            ok = ::ptrGetUserProfileDirectoryW(token, NULL, &dwBufferSize);
+            ok = ptrGetUserProfileDirectoryW(token, NULL, &dwBufferSize);
             if (!ok && dwBufferSize != 0) {        // We got the required buffer size
                 wchar_t *userDirectory = new wchar_t[dwBufferSize];
                 // Second call, now we can fill the allocated buffer.
-                ok = ::ptrGetUserProfileDirectoryW(token, userDirectory, &dwBufferSize);
+                ok = ptrGetUserProfileDirectoryW(token, userDirectory, &dwBufferSize);
                 if (ok)
                     ret = QString::fromWCharArray(userDirectory);
 
@@ -1151,7 +1163,15 @@ bool QFSFileEnginePrivate::doStat() const
         if (filePath.isEmpty())
             return could_stat;
 
-        QString fname = filePath.endsWith(QLatin1String(".lnk")) ? readLink(filePath) : filePath;
+        QString fname;
+        if(filePath.endsWith(QLatin1String(".lnk"))) {
+            fname = readLink(filePath);
+            if(fname.isEmpty())
+                return could_stat;
+        }
+        else
+            fname = filePath;
+
         fname = fixIfRelativeUncPath(fname);
 
         UINT oldmode = SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOOPENFILEERRORBOX);
@@ -1266,12 +1286,7 @@ static QString readSymLink(const QString &link)
         REPARSE_DATA_BUFFER *rdb = (REPARSE_DATA_BUFFER*)qMalloc(bufsize);
         DWORD retsize = 0;
         if (::DeviceIoControl(handle, FSCTL_GET_REPARSE_POINT, 0, 0, rdb, bufsize, &retsize, 0)) {
-            if (rdb->ReparseTag == IO_REPARSE_TAG_MOUNT_POINT) {
-                int length = rdb->MountPointReparseBuffer.SubstituteNameLength / sizeof(wchar_t);
-                int offset = rdb->MountPointReparseBuffer.SubstituteNameOffset / sizeof(wchar_t);
-                const wchar_t* PathBuffer = &rdb->MountPointReparseBuffer.PathBuffer[offset];
-                result = QString::fromWCharArray(PathBuffer, length);
-            } else {
+            if (rdb->ReparseTag == IO_REPARSE_TAG_SYMLINK) {
                 int length = rdb->SymbolicLinkReparseBuffer.SubstituteNameLength / sizeof(wchar_t);
                 int offset = rdb->SymbolicLinkReparseBuffer.SubstituteNameOffset / sizeof(wchar_t);
                 const wchar_t* PathBuffer = &rdb->SymbolicLinkReparseBuffer.PathBuffer[offset];
@@ -1410,33 +1425,28 @@ bool QFSFileEngine::link(const QString &newName)
 #endif // Q_OS_WINCE
 }
 
-/*!
-    \internal
-*/
-QAbstractFileEngine::FileFlags QFSFileEnginePrivate::getPermissions() const
+QAbstractFileEngine::FileFlags QFSFileEnginePrivate::getPermissions(QAbstractFileEngine::FileFlags type) const
 {
     QAbstractFileEngine::FileFlags ret = 0;
 
 #if !defined(QT_NO_LIBRARY)
-    if((qt_ntfs_permission_lookup > 0) && ((QSysInfo::WindowsVersion&QSysInfo::WV_NT_based) > QSysInfo::WV_NT)) {
-        PSID pOwner = 0;
-        PSID pGroup = 0;
-        PACL pDacl;
-        PSECURITY_DESCRIPTOR pSD;
-        ACCESS_MASK access_mask;
-
-        enum { ReadMask = 0x00000001, WriteMask = 0x00000002, ExecMask = 0x00000020 };
+    if((qt_ntfs_permission_lookup > 0) && (QSysInfo::WindowsVersion & QSysInfo::WV_NT_based)) {
         resolveLibs();
         if(ptrGetNamedSecurityInfoW && ptrBuildTrusteeWithSidW && ptrGetEffectiveRightsFromAclW) {
+            enum { ReadMask = 0x00000001, WriteMask = 0x00000002, ExecMask = 0x00000020 };
 
             QString fname = filePath.endsWith(QLatin1String(".lnk")) ? readLink(filePath) : filePath;
+            PSID pOwner = 0;
+            PSID pGroup = 0;
+            PACL pDacl;
+            PSECURITY_DESCRIPTOR pSD;
             DWORD res = ptrGetNamedSecurityInfoW((wchar_t*)fname.utf16(), SE_FILE_OBJECT,
                                                  OWNER_SECURITY_INFORMATION | GROUP_SECURITY_INFORMATION | DACL_SECURITY_INFORMATION,
                                                  &pOwner, &pGroup, &pDacl, 0, &pSD);
-
             if(res == ERROR_SUCCESS) {
+                ACCESS_MASK access_mask;
                 TRUSTEE_W trustee;
-                { //user
+                if (type & 0x0700) { // user
                     if(ptrGetEffectiveRightsFromAclW(pDacl, &currentUserTrusteeW, &access_mask) != ERROR_SUCCESS)
                         access_mask = (ACCESS_MASK)-1;
                     if(access_mask & ReadMask)
@@ -1446,7 +1456,7 @@ QAbstractFileEngine::FileFlags QFSFileEnginePrivate::getPermissions() const
                     if(access_mask & ExecMask)
                         ret |= QAbstractFileEngine::ExeUserPerm;
                 }
-                { //owner
+                if (type & 0x7000) { // owner
                     ptrBuildTrusteeWithSidW(&trustee, pOwner);
                     if(ptrGetEffectiveRightsFromAclW(pDacl, &trustee, &access_mask) != ERROR_SUCCESS)
                         access_mask = (ACCESS_MASK)-1;
@@ -1457,7 +1467,7 @@ QAbstractFileEngine::FileFlags QFSFileEnginePrivate::getPermissions() const
                     if(access_mask & ExecMask)
                         ret |= QAbstractFileEngine::ExeOwnerPerm;
                 }
-                { //group
+                if (type & 0x0070) { // group
                     ptrBuildTrusteeWithSidW(&trustee, pGroup);
                     if(ptrGetEffectiveRightsFromAclW(pDacl, &trustee, &access_mask) != ERROR_SUCCESS)
                         access_mask = (ACCESS_MASK)-1;
@@ -1468,7 +1478,7 @@ QAbstractFileEngine::FileFlags QFSFileEnginePrivate::getPermissions() const
                     if(access_mask & ExecMask)
                         ret |= QAbstractFileEngine::ExeGroupPerm;
                 }
-                { //other (world)
+                if (type & 0x0007) { // other (world)
                     if(ptrGetEffectiveRightsFromAclW(pDacl, &worldTrusteeW, &access_mask) != ERROR_SUCCESS)
                         access_mask = (ACCESS_MASK)-1; // ###
                     if(access_mask & ReadMask)
@@ -1483,29 +1493,35 @@ QAbstractFileEngine::FileFlags QFSFileEnginePrivate::getPermissions() const
         }
     } else
 #endif
-           {
+    {
         //### what to do with permissions if we don't use NTFS
         // for now just add all permissions and what about exe missions ??
-        // also qt_ntfs_permission_lookup is now not set by defualt ... should it ?
-        ret |= QAbstractFileEngine::ReadOtherPerm | QAbstractFileEngine::ReadGroupPerm
-            | QAbstractFileEngine::ReadOwnerPerm | QAbstractFileEngine::ReadUserPerm
-            | QAbstractFileEngine::WriteUserPerm | QAbstractFileEngine::WriteOwnerPerm
-            | QAbstractFileEngine::WriteGroupPerm | QAbstractFileEngine::WriteOtherPerm;
+        // also qt_ntfs_permission_lookup is now not set by default ... should it ?
+        ret |= QAbstractFileEngine::ReadOwnerPerm | QAbstractFileEngine::ReadGroupPerm
+             | QAbstractFileEngine::ReadOtherPerm;
 
-        if (doStat()) {
-            if (ret & (QAbstractFileEngine::WriteOwnerPerm | QAbstractFileEngine::WriteUserPerm |
-                QAbstractFileEngine::WriteGroupPerm | QAbstractFileEngine::WriteOtherPerm)) {
-                if (fileAttrib & FILE_ATTRIBUTE_READONLY)
-                    ret &= ~(QAbstractFileEngine::WriteOwnerPerm | QAbstractFileEngine::WriteUserPerm |
-                             QAbstractFileEngine::WriteGroupPerm | QAbstractFileEngine::WriteOtherPerm);
-            }
+        if (!(fileAttrib & FILE_ATTRIBUTE_READONLY)) {
+            ret |= QAbstractFileEngine::WriteOwnerPerm | QAbstractFileEngine::WriteGroupPerm
+                 | QAbstractFileEngine::WriteOtherPerm;
+        }
 
-            QString fname = filePath.endsWith(QLatin1String(".lnk")) ? readLink(filePath) : filePath;
-            QString ext = fname.right(4).toLower();
-            if (ext == QLatin1String(".exe") || ext == QLatin1String(".com") || ext == QLatin1String(".bat") ||
-                ext == QLatin1String(".pif") || ext == QLatin1String(".cmd") || (fileAttrib & FILE_ATTRIBUTE_DIRECTORY))
-                ret |= QAbstractFileEngine::ExeOwnerPerm | QAbstractFileEngine::ExeGroupPerm |
-                       QAbstractFileEngine::ExeOtherPerm | QAbstractFileEngine::ExeUserPerm;
+        QString fname = filePath.endsWith(QLatin1String(".lnk")) ? readLink(filePath) : filePath;
+        QString ext = fname.right(4).toLower();
+        if ((fileAttrib & FILE_ATTRIBUTE_DIRECTORY) ||
+            ext == QLatin1String(".exe") || ext == QLatin1String(".com") || ext == QLatin1String(".bat") ||
+            ext == QLatin1String(".pif") || ext == QLatin1String(".cmd")) {
+            ret |= QAbstractFileEngine::ExeOwnerPerm | QAbstractFileEngine::ExeGroupPerm
+                 | QAbstractFileEngine::ExeOtherPerm | QAbstractFileEngine::ExeUserPerm;
+        }
+
+        // calculate user permissions
+        if (type & QAbstractFileEngine::ReadUserPerm) {
+            if (::_waccess((wchar_t*)longFileName(fname).utf16(), R_OK) == 0)
+                ret |= QAbstractFileEngine::ReadUserPerm;
+        }
+        if (type & QAbstractFileEngine::WriteUserPerm) {
+            if (::_waccess((wchar_t*)longFileName(fname).utf16(), W_OK) == 0)
+                ret |= QAbstractFileEngine::WriteUserPerm;
         }
     }
     return ret;
@@ -1533,8 +1549,7 @@ bool QFSFileEnginePrivate::isSymlink() const
             if (hFind != INVALID_HANDLE_VALUE) {
                 ::FindClose(hFind);
                 if ((findData.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT)
-                    && (findData.dwReserved0 == IO_REPARSE_TAG_MOUNT_POINT
-                        || findData.dwReserved0 == IO_REPARSE_TAG_SYMLINK)) {
+                    && findData.dwReserved0 == IO_REPARSE_TAG_SYMLINK) {
                     is_link = true;
                 }
             }
@@ -1562,13 +1577,10 @@ QAbstractFileEngine::FileFlags QFSFileEngine::fileFlags(QAbstractFileEngine::Fil
     }
 
     if (type & PermsMask) {
-        ret |= d->getPermissions();
-        // ### Workaround pascals ### above. Since we always set all properties to true
-        // we need to disable read and exec access if the file does not exists
-        if (d->doStat())
+        if (d->doStat()) {
             ret |= ExistsFlag;
-        else
-            ret &= 0x2222;
+            ret |= d->getPermissions(type);
+        }
     }
     if (type & TypesMask) {
         if (d->filePath.endsWith(QLatin1String(".lnk"))) {
@@ -1595,13 +1607,10 @@ QAbstractFileEngine::FileFlags QFSFileEngine::fileFlags(QAbstractFileEngine::Fil
         ret |= LocalDiskFlag;
         if (d->doStat()) {
             ret |= ExistsFlag;
-            if (d->filePath == QLatin1String("/") || isDriveRoot(d->filePath) || isUncRoot(d->filePath)) {
+            if (d->filePath == QLatin1String("/") || isDriveRoot(d->filePath) || isUncRoot(d->filePath))
                 ret |= RootFlag;
-            } else if (d->fileAttrib & FILE_ATTRIBUTE_HIDDEN) {
-                QString baseName = fileName(BaseName);
-                if (baseName != QLatin1String(".") && baseName != QLatin1String(".."))
-                    ret |= HiddenFlag;
-            }
+            else if (d->fileAttrib & FILE_ATTRIBUTE_HIDDEN)
+                ret |= HiddenFlag;
         }
     }
     return ret;
@@ -1723,40 +1732,50 @@ uint QFSFileEngine::ownerId(FileOwner /*own*/) const
 
 QString QFSFileEngine::owner(FileOwner own) const
 {
+    QString name;
 #if !defined(QT_NO_LIBRARY)
     Q_D(const QFSFileEngine);
-    if((qt_ntfs_permission_lookup > 0) && ((QSysInfo::WindowsVersion&QSysInfo::WV_NT_based) > QSysInfo::WV_NT)) {
-	PSID pOwner = 0;
-	PSECURITY_DESCRIPTOR pSD;
-	QString name;
-	QFSFileEnginePrivate::resolveLibs();
-
-	if(ptrGetNamedSecurityInfoW && ptrLookupAccountSidW) {
-	    if(ptrGetNamedSecurityInfoW((wchar_t*)d->filePath.utf16(), SE_FILE_OBJECT,
-					 own == OwnerGroup ? GROUP_SECURITY_INFORMATION : OWNER_SECURITY_INFORMATION,
-					 NULL, &pOwner, NULL, NULL, &pSD) == ERROR_SUCCESS) {
-		DWORD lowner = 0, ldomain = 0;
-		SID_NAME_USE use;
-		// First call, to determine size of the strings (with '\0').
-		ptrLookupAccountSidW(NULL, pOwner, NULL, &lowner, NULL, &ldomain, (SID_NAME_USE*)&use);
-		wchar_t *owner = new wchar_t[lowner];
-		wchar_t *domain = new wchar_t[ldomain];
-		// Second call, size is without '\0'
-		if(ptrLookupAccountSidW(NULL, pOwner, (LPWSTR)owner, &lowner,
-					 (LPWSTR)domain, &ldomain, (SID_NAME_USE*)&use)) {
-		    name = QString::fromUtf16((ushort*)owner);
-		}
-		LocalFree(pSD);
-		delete [] owner;
-		delete [] domain;
-	    }
-	}
-	return name;
+    if((qt_ntfs_permission_lookup > 0) && (QSysInfo::WindowsVersion & QSysInfo::WV_NT_based)) {
+        QFSFileEnginePrivate::resolveLibs();
+        if (ptrGetNamedSecurityInfoW && ptrLookupAccountSidW) {
+            PSID pOwner = 0;
+            PSECURITY_DESCRIPTOR pSD;
+            if (ptrGetNamedSecurityInfoW((wchar_t*)d->filePath.utf16(), SE_FILE_OBJECT,
+                                         own == OwnerGroup ? GROUP_SECURITY_INFORMATION : OWNER_SECURITY_INFORMATION,
+                                         own == OwnerUser ? &pOwner : 0, own == OwnerGroup ? &pOwner : 0,
+                                         0, 0, &pSD) == ERROR_SUCCESS) {
+                DWORD lowner = 64;
+                DWORD ldomain = 64;
+                QVarLengthArray<wchar_t, 64> owner(lowner);
+                QVarLengthArray<wchar_t, 64> domain(ldomain);
+                SID_NAME_USE use = SidTypeUnknown;
+                // First call, to determine size of the strings (with '\0').
+                if (!ptrLookupAccountSidW(NULL, pOwner, (LPWSTR)owner.data(), &lowner,
+                                          (LPWSTR)domain.data(), &ldomain, (SID_NAME_USE*)&use)) {
+                    if (GetLastError() == ERROR_INSUFFICIENT_BUFFER) {
+                        if (lowner > (DWORD)owner.size())
+                            owner.resize(lowner);
+                        if (ldomain > (DWORD)domain.size())
+                            domain.resize(ldomain);
+                        // Second call, try on resized buf-s
+                        if (!ptrLookupAccountSidW(NULL, pOwner, (LPWSTR)owner.data(), &lowner,
+                                                  (LPWSTR)domain.data(), &ldomain, (SID_NAME_USE*)&use)) {
+                            lowner = 0;
+                        }
+                    } else {
+                        lowner = 0;
+                    }
+                }
+                if (lowner != 0)
+                    name = QString::fromWCharArray(owner.data());
+                LocalFree(pSD);
+            }
+        }
     }
 #else
     Q_UNUSED(own);
 #endif
-    return QString();
+    return name;
 }
 
 bool QFSFileEngine::setPermissions(uint perms)
@@ -1940,6 +1959,10 @@ uchar *QFSFileEnginePrivate::map(qint64 offset, qint64 size,
                 OPEN_EXISTING,
                 FILE_ATTRIBUTE_NORMAL,
                 NULL);
+        // Since this is a special case, we check if the return value was NULL and if so
+        // we change it to INVALID_HANDLE_VALUE to follow the logic inside this function.
+        if(0 == handle)
+            handle = INVALID_HANDLE_VALUE;
 #endif
 
         if (handle == INVALID_HANDLE_VALUE) {
