@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -47,7 +47,6 @@
 #include "qeventloop.h"
 #include "qcorecmdlineargs_p.h"
 #include <qdatastream.h>
-#include <qdatetime.h>
 #include <qdebug.h>
 #include <qdir.h>
 #include <qfile.h>
@@ -59,14 +58,17 @@
 #include <qthreadpool.h>
 #include <qthreadstorage.h>
 #include <private/qthread_p.h>
+#include <qelapsedtimer.h>
 #include <qlibraryinfo.h>
 #include <qvarlengtharray.h>
 #include <private/qfactoryloader_p.h>
 #include <private/qfunctions_p.h>
+#include <private/qlocale_p.h>
 
 #ifdef Q_OS_SYMBIAN
 #  include <exception>
 #  include <f32file.h>
+#  include <e32ldr.h>
 #  include "qeventdispatcher_symbian_p.h"
 #  include "private/qcore_symbian_p.h"
 #elif defined(Q_OS_UNIX)
@@ -513,17 +515,18 @@ QCoreApplication::QCoreApplication(int &argc, char **argv)
 {
     init();
     QCoreApplicationPrivate::eventDispatcher->startingUp();
-#if defined(Q_OS_SYMBIAN) && !defined(QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
+#if defined(Q_OS_SYMBIAN) && !defined(QT_NO_LIBRARY)
     // Refresh factoryloader, as text codecs are requested during lib path
     // resolving process and won't be therefore properly loaded.
     // Unknown if this is symbian specific issue.
     QFactoryLoader::refreshAll();
 #endif
 
+#if defined(Q_OS_SYMBIAN) && !defined(QT_NO_SYSTEMLOCALE)
+    d_func()->symbianInit();
+#endif
 }
 
-extern void set_winapp_name();
-
 // ### move to QCoreApplicationPrivate constructor?
 void QCoreApplication::init()
 {
@@ -534,11 +537,6 @@ void QCoreApplication::init()
     qt_locale_initialized = true;
 #endif
 
-#ifdef Q_WS_WIN
-    // Get the application name/instance if qWinMain() was not invoked
-    set_winapp_name();
-#endif
-
     Q_ASSERT_X(!self, "QCoreApplication", "there should be only one application object");
     QCoreApplication::self = this;
 
@@ -579,9 +577,39 @@ void QCoreApplication::init()
     qt_core_eval_init(d->application_type);
 #endif
 
+#if    defined(Q_OS_SYMBIAN)  \
+    && defined(Q_CC_NOKIAX86) \
+    && defined(QT_DEBUG)
+    /**
+     * Prevent the executable from being locked in the Symbian emulator. The
+     * code dramatically simplifies debugging on Symbian, but beyond that has
+     * no impact.
+     *
+     * Force the ZLazyUnloadTimer to fire and therefore unload code segments
+     * immediately. The code affects Symbian's file server and on the other
+     * hand needs only to be run once in each emulator run.
+     */
+    {
+        RLoader loader;
+        CleanupClosePushL(loader);
+        User::LeaveIfError(loader.Connect());
+        User::LeaveIfError(loader.CancelLazyDllUnload());
+        CleanupStack::PopAndDestroy(&loader);
+    }
+#endif
+
     qt_startup_hook();
 }
 
+#if defined(Q_OS_SYMBIAN) && !defined(QT_NO_SYSTEMLOCALE)
+void QCoreApplicationPrivate::symbianInit()
+{
+    if (!environmentChangeNotifier)
+        environmentChangeNotifier.reset(new QEnvironmentChangeNotifier);
+}
+#endif
+
+
 /*!
     Destroys the QCoreApplication object.
 */
@@ -917,7 +945,7 @@ void QCoreApplication::processEvents(QEventLoop::ProcessEventsFlags flags, int m
     QThreadData *data = QThreadData::current();
     if (!data->eventDispatcher)
         return;
-    QTime start;
+    QElapsedTimer start;
     start.start();
     if (flags & QEventLoop::DeferredDeletion)
         QCoreApplication::sendPostedEvents(0, QEvent::DeferredDelete);
@@ -1045,7 +1073,7 @@ void QCoreApplication::exit(int returnCode)
 
     The event must be allocated on the heap since the post event queue
     will take ownership of the event and delete it once it has been
-    posted.  It is \e {not safe} to modify or delete the event after
+    posted.  It is \e {not safe} to access the event after
     it has been posted.
 
     When control returns to the main event loop, all events that are
@@ -1076,7 +1104,7 @@ void QCoreApplication::postEvent(QObject *receiver, QEvent *event)
 
     The event must be allocated on the heap since the post event queue
     will take ownership of the event and delete it once it has been
-    posted.  It is \e {not safe} to modify or delete the event after
+    posted.  It is \e {not safe} to access the event after
     it has been posted.
 
     When control returns to the main event loop, all events that are
@@ -2063,7 +2091,8 @@ QStringList QCoreApplication::arguments()
                 l1arg == "-stylesheet" ||
                 l1arg == "-widgetcount")
                 ;
-            else if (l1arg.startsWith("-style="))
+            else if (l1arg.startsWith("-style=") ||
+                     l1arg.startsWith("-qmljsdebugger="))
                 ;
             else if (l1arg == "-style" ||
                      l1arg == "-session" ||
@@ -2174,7 +2203,7 @@ QString QCoreApplication::applicationVersion()
     return coreappdata()->applicationVersion;
 }
 
-#if !defined(QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
+#ifndef QT_NO_LIBRARY
 
 Q_GLOBAL_STATIC_WITH_ARGS(QMutex, libraryPathMutex, (QMutex::Recursive))
 
@@ -2219,7 +2248,8 @@ QStringList QCoreApplication::libraryPaths()
             TFindFile finder(fs);
             TInt err = finder.FindByDir(tempPathPtr, tempPathPtr);
             while (err == KErrNone) {
-                QString foundDir = QString::fromUtf16(finder.File().Ptr(), finder.File().Length());
+                QString foundDir(reinterpret_cast<const QChar *>(finder.File().Ptr()),
+                                 finder.File().Length());
                 foundDir = QDir(foundDir).canonicalPath();
                 if (!app_libpaths->contains(foundDir))
                     app_libpaths->append(foundDir);
@@ -2267,17 +2297,20 @@ QStringList QCoreApplication::libraryPaths()
     \a paths. All existing paths will be deleted and the path list
     will consist of the paths given in \a paths.
 
+    In Symbian this function is only useful for setting paths for
+    finding Qt extension plugin stubs, since the OS can only
+    load libraries from the \c{/sys/bin} directory.
+
     \sa libraryPaths(), addLibraryPath(), removeLibraryPath(), QLibrary
  */
 void QCoreApplication::setLibraryPaths(const QStringList &paths)
 {
-#if !defined(QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
     QMutexLocker locker(libraryPathMutex());
     if (!coreappdata()->app_libpaths)
         coreappdata()->app_libpaths = new QStringList;
     *(coreappdata()->app_libpaths) = paths;
+    locker.unlock();
     QFactoryLoader::refreshAll();
-#endif
 }
 
 /*!
@@ -2290,11 +2323,14 @@ void QCoreApplication::setLibraryPaths(const QStringList &paths)
   is \c INSTALL/plugins, where \c INSTALL is the directory where Qt was
   installed.
 
+  In Symbian this function is only useful for adding paths for
+  finding Qt extension plugin stubs, since the OS can only
+  load libraries from the \c{/sys/bin} directory.
+
   \sa removeLibraryPath(), libraryPaths(), setLibraryPaths()
  */
 void QCoreApplication::addLibraryPath(const QString &path)
 {
-#if !defined(QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
     if (path.isEmpty())
         return;
 
@@ -2307,9 +2343,9 @@ void QCoreApplication::addLibraryPath(const QString &path)
     if (!canonicalPath.isEmpty()
         && !coreappdata()->app_libpaths->contains(canonicalPath)) {
         coreappdata()->app_libpaths->prepend(canonicalPath);
+        locker.unlock();
         QFactoryLoader::refreshAll();
     }
-#endif
 }
 
 /*!
@@ -2320,7 +2356,6 @@ void QCoreApplication::addLibraryPath(const QString &path)
 */
 void QCoreApplication::removeLibraryPath(const QString &path)
 {
-#if !defined(QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
     if (path.isEmpty())
         return;
 
@@ -2332,7 +2367,6 @@ void QCoreApplication::removeLibraryPath(const QString &path)
     QString canonicalPath = QDir(path).canonicalPath();
     coreappdata()->app_libpaths->removeAll(canonicalPath);
     QFactoryLoader::refreshAll();
-#endif
 }
 
 #endif //QT_NO_LIBRARY
