@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -91,6 +91,16 @@
     animation.setDuration(1000);
     animation.setEasingCurve(QEasingCurve::InOutQuad);
     \endcode
+
+    The ability to set an amplitude, overshoot, or period depends on the QEasingCurve type. Amplitude access
+    is available to curves that behave as springs such as elastic and bounce curves. Changing the amplitude changes
+    the height of the curve. Period access is only available to elastic curves and setting a higher period slows
+    the rate of bounce. Only curves that have "boomerang" behaviors such as the InBack, OutBack, InOutBack, and OutInBack 
+    have overshoot settings. These curves will interpolate beyond the end points and return to the end point, 
+    acting similar to a boomerang.
+
+    The \l{Easing Curves Example} contains samples of QEasingCurve types and lets you change the curve settings.
+
  */
 
 /*!
@@ -125,7 +135,7 @@
     \value OutCubic     \inlineimage qeasingcurve-outcubic.png
                         \br
                         Easing curve for a cubic (t^3) function:
-                        decelerating from zero velocity.
+                        decelerating to zero velocity.
     \value InOutCubic   \inlineimage qeasingcurve-inoutcubic.png
                         \br
                         Easing curve for a cubic (t^3) function:
@@ -140,15 +150,15 @@
                         accelerating from zero velocity.
     \value OutQuart     \inlineimage qeasingcurve-outquart.png
                         \br
-                        Easing curve for a cubic (t^4) function:
-                        decelerating from zero velocity.
+                        Easing curve for a quartic (t^4) function:
+                        decelerating to zero velocity.
     \value InOutQuart   \inlineimage qeasingcurve-inoutquart.png
                         \br
-                        Easing curve for a cubic (t^4) function:
+                        Easing curve for a quartic (t^4) function:
                         acceleration until halfway, then deceleration.
     \value OutInQuart   \inlineimage qeasingcurve-outinquart.png
                         \br
-                        Easing curve for a cubic (t^4) function:
+                        Easing curve for a quartic (t^4) function:
                         deceleration until halfway, then acceleration.
     \value InQuint      \inlineimage qeasingcurve-inquint.png
                         \br
@@ -156,15 +166,15 @@
                         in: accelerating from zero velocity.
     \value OutQuint     \inlineimage qeasingcurve-outquint.png
                         \br
-                        Easing curve for a cubic (t^5) function:
-                        decelerating from zero velocity.
+                        Easing curve for a quintic (t^5) function:
+                        decelerating to zero velocity.
     \value InOutQuint   \inlineimage qeasingcurve-inoutquint.png
                         \br
-                        Easing curve for a cubic (t^5) function:
+                        Easing curve for a quintic (t^5) function:
                         acceleration until halfway, then deceleration.
     \value OutInQuint   \inlineimage qeasingcurve-outinquint.png
                         \br
-                        Easing curve for a cubic (t^5) function:
+                        Easing curve for a quintic (t^5) function:
                         deceleration until halfway, then acceleration.
     \value InSine       \inlineimage qeasingcurve-insine.png
                         \br
@@ -304,6 +314,10 @@
 #include <QtCore/qstring.h>
 #endif
 
+#ifndef QT_NO_DATASTREAM
+#include <QtCore/qdatastream.h>
+#endif
+
 QT_BEGIN_NAMESPACE
 
 static bool isConfigFunction(QEasingCurve::Type type)
@@ -318,7 +332,7 @@ public:
     enum Type { In, Out, InOut, OutIn };
 
     QEasingCurveFunction(QEasingCurveFunction::Type type = In, qreal period = 0.3, qreal amplitude = 1.0,
-        qreal overshoot = 1.70158f)
+        qreal overshoot = 1.70158)
         : _t(type), _p(period), _a(amplitude), _o(overshoot)
     { }
     virtual ~QEasingCurveFunction() {}
@@ -345,9 +359,9 @@ QEasingCurveFunction *QEasingCurveFunction::copy() const
 bool QEasingCurveFunction::operator==(const QEasingCurveFunction& other)
 {
     return _t == other._t &&
-           _p == other._p &&
-           _a == other._a &&
-           _o == other._o;
+           qFuzzyCompare(_p, other._p) &&
+           qFuzzyCompare(_a, other._a) &&
+           qFuzzyCompare(_o, other._o);
 }
 
 QT_BEGIN_INCLUDE_NAMESPACE
@@ -386,8 +400,8 @@ struct ElasticEase : public QEasingCurveFunction
 
     qreal value(qreal t)
     {
-        qreal p = (_p < 0) ? 0.3f : _p;
-        qreal a = (_a < 0) ? 1.0f : _a;
+        qreal p = (_p < 0) ? qreal(0.3) : _p;
+        qreal a = (_a < 0) ? qreal(1.0) : _a;
         switch(_t) {
         case In:
             return easeInElastic(t, a, p);
@@ -406,7 +420,7 @@ struct ElasticEase : public QEasingCurveFunction
 struct BounceEase : public QEasingCurveFunction
 {
     BounceEase(Type type)
-        : QEasingCurveFunction(type, 0.3f, 1.0f)
+        : QEasingCurveFunction(type, qreal(0.3), qreal(1.0))
     { }
 
     QEasingCurveFunction *copy() const
@@ -418,7 +432,7 @@ struct BounceEase : public QEasingCurveFunction
 
     qreal value(qreal t)
     {
-        qreal a = (_a < 0) ? 1.0f : _a;
+        qreal a = (_a < 0) ? qreal(1.0) : _a;
         switch(_t) {
         case In:
             return easeInBounce(t, a);
@@ -437,7 +451,7 @@ struct BounceEase : public QEasingCurveFunction
 struct BackEase : public QEasingCurveFunction
 {
     BackEase(Type type)
-        : QEasingCurveFunction(type, 0.3f, 1.0f, 1.70158f)
+        : QEasingCurveFunction(type, qreal(0.3), qreal(1.0), qreal(1.70158))
     { }
 
     QEasingCurveFunction *copy() const
@@ -449,7 +463,7 @@ struct BackEase : public QEasingCurveFunction
 
     qreal value(qreal t)
     {
-        qreal o = (_o < 0) ? 1.70158f : _o;
+        qreal o = (_o < 0) ? qreal(1.70158) : _o;
         switch(_t) {
         case In:
             return easeInBack(t, o);
@@ -581,7 +595,7 @@ static QEasingCurveFunction *curveToFunctionObject(QEasingCurve::Type type)
         curveFunc = new BackEase(BackEase::OutIn);
         break;
     default:
-        curveFunc = new QEasingCurveFunction(QEasingCurveFunction::In, 0.3f, 1.0f, 1.70158f);     // ###
+        curveFunc = new QEasingCurveFunction(QEasingCurveFunction::In, qreal(0.3), qreal(1.0), qreal(1.70158));
     }
 
     return curveFunc;
@@ -600,11 +614,11 @@ QEasingCurve::QEasingCurve(Type type)
     Construct a copy of \a other.
  */
 QEasingCurve::QEasingCurve(const QEasingCurve &other)
-: d_ptr(new QEasingCurvePrivate)
+    : d_ptr(new QEasingCurvePrivate)
 {
     // ### non-atomic, requires malloc on shallow copy
     *d_ptr = *other.d_ptr;
-    if(other.d_ptr->config)
+    if (other.d_ptr->config)
         d_ptr->config = other.d_ptr->config->copy();
 }
 
@@ -629,7 +643,7 @@ QEasingCurve &QEasingCurve::operator=(const QEasingCurve &other)
     }
 
     *d_ptr = *other.d_ptr;
-    if(other.d_ptr->config)
+    if (other.d_ptr->config)
         d_ptr->config = other.d_ptr->config->copy();
 
     return *this;
@@ -643,9 +657,17 @@ bool QEasingCurve::operator==(const QEasingCurve &other) const
 {
     bool res = d_ptr->func == other.d_ptr->func
             && d_ptr->type == other.d_ptr->type;
-    if (res && d_ptr->config && other.d_ptr->config) {
+    if (res) {
+        if (d_ptr->config && other.d_ptr->config) {
         // catch the config content
-        res = d_ptr->config->operator==(*(other.d_ptr->config));
+            res = d_ptr->config->operator==(*(other.d_ptr->config));
+
+        } else if (d_ptr->config || other.d_ptr->config) {
+        // one one has a config object, which could contain default values
+            res = qFuzzyCompare(amplitude(), other.amplitude()) &&
+                  qFuzzyCompare(period(), other.period()) &&
+                  qFuzzyCompare(overshoot(), other.overshoot());
+        }
     }
     return res;
 }
@@ -667,7 +689,7 @@ bool QEasingCurve::operator==(const QEasingCurve &other) const
  */
 qreal QEasingCurve::amplitude() const
 {
-    return d_ptr->config ? d_ptr->config->_a : 1.0;
+    return d_ptr->config ? d_ptr->config->_a : qreal(1.0);
 }
 
 /*!
@@ -691,7 +713,7 @@ void QEasingCurve::setAmplitude(qreal amplitude)
  */
 qreal QEasingCurve::period() const
 {
-    return d_ptr->config ? d_ptr->config->_p : 0.3;
+    return d_ptr->config ? d_ptr->config->_p : qreal(0.3);
 }
 
 /*!
@@ -715,7 +737,7 @@ void QEasingCurve::setPeriod(qreal period)
  */
 qreal QEasingCurve::overshoot() const
 {
-    return d_ptr->config ? d_ptr->config->_o : 1.70158f;
+    return d_ptr->config ? d_ptr->config->_o : qreal(1.70158) ;
 }
 
 /*!
@@ -845,6 +867,67 @@ QDebug operator<<(QDebug debug, const QEasingCurve &item)
     }
     return debug;
 }
-#endif
+#endif // QT_NO_DEBUG_STREAM
+
+#ifndef QT_NO_DATASTREAM
+/*!
+    \fn QDataStream &operator<<(QDataStream &stream, const QEasingCurve &easing)
+    \relates QEasingCurve
+
+    Writes the given \a easing curve to the given \a stream and returns a
+    reference to the stream.
+
+    \sa {Serializing Qt Data Types}
+*/
+
+QDataStream &operator<<(QDataStream &stream, const QEasingCurve &easing)
+{
+    stream << quint8(easing.d_ptr->type);
+    stream << quint64(quintptr(easing.d_ptr->func));
+
+    bool hasConfig = easing.d_ptr->config;
+    stream << hasConfig;
+    if (hasConfig) {
+        stream << easing.d_ptr->config->_p;
+        stream << easing.d_ptr->config->_a;
+        stream << easing.d_ptr->config->_o;
+    }
+    return stream;
+}
+
+/*!
+    \fn QDataStream &operator>>(QDataStream &stream, QEasingCurve &easing)
+    \relates QQuaternion
+
+    Reads an easing curve from the given \a stream into the given \a
+    easing curve and returns a reference to the stream.
+
+    \sa {Serializing Qt Data Types}
+*/
+
+QDataStream &operator>>(QDataStream &stream, QEasingCurve &easing)
+{
+    QEasingCurve::Type type;
+    quint8 int_type;
+    stream >> int_type;
+    type = static_cast<QEasingCurve::Type>(int_type);
+    easing.setType(type);
+
+    quint64 ptr_func;
+    stream >> ptr_func;
+    easing.d_ptr->func = QEasingCurve::EasingFunction(quintptr(ptr_func));
+
+    bool hasConfig;
+    stream >> hasConfig;
+    if (hasConfig) {
+        QEasingCurveFunction *config = curveToFunctionObject(type);
+        stream >> config->_p;
+        stream >> config->_a;
+        stream >> config->_o;
+        easing.d_ptr->config = config;
+    }
+    return stream;
+}
+#endif // QT_NO_DATASTREAM
 
 QT_END_NAMESPACE
