@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -54,6 +54,8 @@
 
 #include "qdbusthreaddebug_p.h"
 
+#ifndef QT_NO_DBUS
+
 QT_BEGIN_NAMESPACE
 
 class QDBusConnectionManager
@@ -123,7 +125,7 @@ QDBusConnectionManager::~QDBusConnectionManager()
     connectionHash.clear();
 }
 
-QDBUS_EXPORT void qDBusBindToApplication();
+Q_DBUS_EXPORT void qDBusBindToApplication();
 void qDBusBindToApplication()
 {
 }
@@ -171,6 +173,9 @@ void QDBusConnectionManager::setConnection(const QString &name, QDBusConnectionP
     The connection is then torn down using the disconnectFromBus()
     function.
 
+    Once disconnected, calling connectToBus() will not reestablish a
+    connection, you must create a new QDBusConnection instance.
+
     As a convenience for the two most common connection types, the
     sessionBus() and systemBus() functions return open connections to
     the session server daemon and the system server daemon,
@@ -207,6 +212,7 @@ void QDBusConnectionManager::setConnection(const QString &name, QDBusConnectionP
     \value ExportScriptableSlots                export this object's scriptable slots
     \value ExportScriptableSignals              export this object's scriptable signals
     \value ExportScriptableProperties           export this object's scriptable properties
+    \value ExportScriptableInvokables           export this object's scriptable invokables
     \value ExportScriptableContents             shorthand form for ExportScriptableSlots |
                                                 ExportScriptableSignals |
                                                 ExportScriptableProperties
@@ -214,6 +220,7 @@ void QDBusConnectionManager::setConnection(const QString &name, QDBusConnectionP
     \value ExportNonScriptableSlots             export this object's non-scriptable slots
     \value ExportNonScriptableSignals           export this object's non-scriptable signals
     \value ExportNonScriptableProperties        export this object's non-scriptable properties
+    \value ExportNonScriptableInvokables        export this object's non-scriptable invokables
     \value ExportNonScriptableContents          shorthand form for ExportNonScriptableSlots |
                                                 ExportNonScriptableSignals |
                                                 ExportNonScriptableProperties
@@ -221,8 +228,8 @@ void QDBusConnectionManager::setConnection(const QString &name, QDBusConnectionP
     \value ExportAllSlots                       export all of this object's slots
     \value ExportAllSignals                     export all of this object's signals
     \value ExportAllProperties                  export all of this object's properties
+    \value ExportAllInvokables                  export all of this object's invokables
     \value ExportAllContents                    export all of this object's contents
-
     \value ExportChildObjects                   export this object's child objects
 
     \sa registerObject(), QDBusAbstractAdaptor, {usingadaptors.html}{Using adaptors}
@@ -491,6 +498,12 @@ bool QDBusConnection::callWithCallback(const QDBusMessage &message, QObject *rec
     its return value, which will be either of type
     QDBusMessage::ReplyMessage or QDBusMessage::ErrorMessage.
 
+    If no reply is received within \a timeout milliseconds, an automatic
+    error will be delivered indicating the expiration of the call.
+    The default \a timeout is -1, which will be replaced with an
+    implementation-defined value that is suitable for inter-process
+    communications (generally, 25 seconds).
+
     See the QDBusInterface::call() function for a more friendly way
     of placing calls.
 
@@ -526,9 +539,14 @@ QDBusMessage QDBusConnection::call(const QDBusMessage &message, QDBus::CallMode
     Sends the \a message over this connection and returns
     immediately. This function is suitable for method calls only. It
     returns an object of type QDBusPendingCall which can be used to
-    track the status of the reply. The \a timeout parameter is used to
-    determine when an auto-generated error reply may be emitted and is
-    also the upper limit for waiting in QDBusPendingCall::waitForFinished().
+    track the status of the reply.
+
+    If no reply is received within \a timeout milliseconds, an automatic
+    error will be delivered indicating the expiration of the call. The
+    default \a timeout is -1, which will be replaced with an
+    implementation-defined value that is suitable for inter-process
+    communications (generally, 25 seconds). This timeout is also the
+    upper limit for waiting in QDBusPendingCall::waitForFinished().
 
     See the QDBusInterface::asyncCall() function for a more friendly way
     of placing calls.
@@ -619,10 +637,8 @@ bool QDBusConnection::connect(const QString &service, const QString &path, const
     if (interface.isEmpty() && name.isEmpty())
         return false;
 
-    QString owner = d->getNameOwner(service); // we don't care if the owner is empty
-                                              // it might get started later
     QDBusWriteLocker locker(ConnectAction, d);
-    return d->connectSignal(service, owner, path, interface, name, argumentMatch, signature, receiver, slot);
+    return d->connectSignal(service, path, interface, name, argumentMatch, signature, receiver, slot);
 }
 
 /*!
@@ -842,10 +858,6 @@ QDBusConnectionInterface *QDBusConnection::interface() const
 
 /*!
     Returns true if this QDBusConnection object is connected.
-
-    If it isn't connected, calling connectToBus() on the same
-    connection name will not make be connected. You need to call the
-    QDBusConnection constructor again.
 */
 bool QDBusConnection::isConnected() const
 {
@@ -905,7 +917,7 @@ QString QDBusConnection::name() const
 
 /*!
     Attempts to register the \a serviceName on the D-Bus server and
-    returns true if the registration succeded. The registration will
+    returns true if the registration succeeded. The registration will
     fail if the name is already registered by another application.
 
     \sa unregisterService(), QDBusConnectionInterface::registerService()
@@ -989,14 +1001,6 @@ void QDBusConnectionPrivate::setSender(const QDBusConnectionPrivate *s)
     _q_manager()->setSender(s);
 }
 
-/*!
-  \internal
-*/
-void QDBusConnectionPrivate::setConnection(const QString &name, QDBusConnectionPrivate *c)
-{
-    _q_manager()->setConnection(name, c);
-}
-
 /*!
   \internal
 */
@@ -1005,9 +1009,6 @@ void QDBusConnectionPrivate::setBusService(const QDBusConnection &connection)
     busService = new QDBusConnectionInterface(connection, this);
     ref.deref(); // busService has increased the refcounting to us
                  // avoid cyclic refcounting
-//    if (mode != PeerMode)
-    QObject::connect(busService, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
-                     this, SIGNAL(serviceOwnerChanged(QString,QString,QString)));
 
     QObject::connect(this, SIGNAL(callWithCallbackFailed(QDBusError,QDBusMessage)),
                      busService, SIGNAL(callWithCallbackFailed(QDBusError,QDBusMessage)),
@@ -1042,3 +1043,5 @@ void QDBusConnectionPrivate::setBusService(const QDBusConnection &connection)
 */
 
 QT_END_NAMESPACE
+
+#endif // QT_NO_DBUS
