@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -81,8 +81,10 @@ static QString qt_strippedText(QString s)
 
 QActionPrivate::QActionPrivate() : group(0), enabled(1), forceDisabled(0),
                                    visible(1), forceInvisible(0), checkable(0), checked(0), separator(0), fontSet(false),
+                                   forceEnabledInSoftkeys(false), menuActionSoftkeys(false),
+                                   iconVisibleInMenu(-1),
                                    menuRole(QAction::TextHeuristicRole), softKeyRole(QAction::NoSoftKey),
-                                   priority(QAction::NormalPriority), iconVisibleInMenu(-1)
+                                   priority(QAction::NormalPriority)
 {
 #ifdef QT3_SUPPORT
     static int qt_static_action_id = -1;
@@ -100,6 +102,21 @@ QActionPrivate::~QActionPrivate()
 {
 }
 
+bool QActionPrivate::showStatusText(QWidget *widget, const QString &str)
+{
+#ifdef QT_NO_STATUSTIP
+    Q_UNUSED(widget);
+    Q_UNUSED(str);
+#else
+    if(QObject *object = widget ? widget : parent) {
+        QStatusTipEvent tip(str);
+        QApplication::sendEvent(object, &tip);
+        return true;
+    }
+#endif
+    return false;
+}
+
 void QActionPrivate::sendDataChanged()
 {
     Q_Q(QAction);
@@ -700,6 +717,10 @@ QActionGroup *QAction::actionGroup() const
     it is displayed to the left of the menu text. There is no default
     icon.
 
+    On Symbian the icons which are passed to softkeys, i.e. to actions with
+    softkey role, need to have pixmap alpha channel correctly set otherwise
+    drawing artifacts will appear when softkey is pressed down.
+
     If a null icon (QIcon::isNull() is passed into this function,
     the icon of the action is cleared.
 */
@@ -1206,16 +1227,7 @@ QAction::setData(const QVariant &data)
 bool
 QAction::showStatusText(QWidget *widget)
 {
-#ifdef QT_NO_STATUSTIP
-    Q_UNUSED(widget);
-#else
-    if(QObject *object = widget ? widget : parent()) {
-        QStatusTipEvent tip(statusTip());
-        QApplication::sendEvent(object, &tip);
-        return true;
-    }
-#endif
-    return false;
+    return d_func()->showStatusText(widget, statusTip());
 }
 
 /*!
