@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -65,7 +65,6 @@
 #include "qapplication.h"
 #include "qdesktopwidget.h"
 #include "qbitmap.h"
-#include "qdatetime.h"
 #include "qiodevice.h"
 #include "qbuffer.h"
 #include "qtextcodec.h"
@@ -76,6 +75,7 @@
 #include "qt_x11_p.h"
 #include "qx11info_x11.h"
 #include "qimagewriter.h"
+#include "qelapsedtimer.h"
 #include "qvariant.h"
 #include "qdnd_p.h"
 #include <private/qwidget_p.h>
@@ -456,7 +456,8 @@ QClipboard::QClipboard(QObject *parent)
         XCheckIfEvent(X11->display, &ev, &qt_init_timestamp_scanner, (XPointer)&data);
         if (data.timestamp == CurrentTime) {
             setupOwner();
-            int dummy = 0;
+            // We need this value just for completeness, we don't use it.
+            long dummy = 0;
             Window ownerId = owner->internalWinId();
             XChangeProperty(X11->display, ownerId,
                             ATOM(CLIP_TEMPORARY), XA_INTEGER, 32,
@@ -516,8 +517,9 @@ static Bool checkForClipboardEvents(Display *, XEvent *e, XPointer)
 
 bool QX11Data::clipboardWaitForEvent(Window win, int type, XEvent *event, int timeout)
 {
-    QTime started = QTime::currentTime();
-    QTime now = started;
+    QElapsedTimer started;
+    started.start();
+    QElapsedTimer now = started;
 
     if (QAbstractEventDispatcher::instance()->inherits("QtMotif")
         || QApplication::clipboard()->property("useEventLoopWhenWaiting").toBool()) {
@@ -545,9 +547,7 @@ bool QX11Data::clipboardWaitForEvent(Window win, int type, XEvent *event, int ti
             XSync(X11->display, false);
             usleep(50000);
 
-            now = QTime::currentTime();
-            if (started > now)                        // crossed midnight
-                started = now;
+            now.start();
 
             QEventLoop::ProcessEventsFlags flags(QEventLoop::ExcludeUserInputEvents
                                                  | QEventLoop::ExcludeSocketNotifiers
@@ -576,9 +576,7 @@ bool QX11Data::clipboardWaitForEvent(Window win, int type, XEvent *event, int ti
             if (XCheckIfEvent(X11->display, &e, checkForClipboardEvents, 0))
                 qApp->x11ProcessEvent(&e);
 
-            now = QTime::currentTime();
-            if ( started > now )                        // crossed midnight
-                started = now;
+            now.start();
 
             XFlush(X11->display);
 
@@ -1521,7 +1519,7 @@ bool qt_xfixes_selection_changed(Window selectionOwner, Time timestamp)
           (unsigned int)(d ? d->timestamp : 0), (unsigned int)timestamp);
 #endif
     if (!owner || (selectionOwner && selectionOwner != owner->internalWinId()) ||
-        (!selectionOwner && d->timestamp != CurrentTime && d->timestamp < timestamp))
+        (!selectionOwner && (d->timestamp == CurrentTime || d->timestamp < timestamp)))
         return qt_check_selection_sentinel();
     return false;
 }
@@ -1535,7 +1533,7 @@ bool qt_xfixes_clipboard_changed(Window clipboardOwner, Time timestamp)
           (unsigned int)(d ? d->timestamp : 0), (unsigned int)timestamp);
 #endif
     if (!owner || (clipboardOwner && clipboardOwner != owner->internalWinId()) ||
-        (!clipboardOwner && d->timestamp != CurrentTime && d->timestamp < timestamp))
+        (!clipboardOwner && (d->timestamp == CurrentTime || d->timestamp < timestamp)))
         return qt_check_clipboard_sentinel();
     return false;
 }
