@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -45,6 +45,7 @@
 #include "private/qgesture_p.h"
 #include "private/qgraphicsitem_p.h"
 #include "private/qevent_p.h"
+#include "private/qapplication_p.h"
 #include "qgesture.h"
 #include "qevent.h"
 #include "qgraphicsitem.h"
@@ -65,19 +66,12 @@
 # define DEBUG qDebug
 #endif
 
+#ifndef QT_NO_GESTURES
+
 QT_BEGIN_NAMESPACE
 
-QGestureManager *qt_gestureManager = 0;
-
-QGestureManager* QGestureManager::instance()
-{
-    if (!qt_gestureManager)
-        qt_gestureManager = new QGestureManager(qApp);
-    return qt_gestureManager;
-}
-
 QGestureManager::QGestureManager(QObject *parent)
-    : QObject(parent), state(NotGesture), m_lastCustomGestureId(0)
+    : QObject(parent), state(NotGesture), m_lastCustomGestureId(Qt::CustomGesture)
 {
     qRegisterMetaType<Qt::GestureState>();
 
@@ -95,7 +89,8 @@ QGestureManager::QGestureManager(QObject *parent)
 #endif
 #if defined(Q_OS_WIN)
   #if !defined(QT_NO_NATIVE_GESTURES)
-    registerGestureRecognizer(new QWinNativePanGestureRecognizer);
+    if (QApplicationPrivate::HasTouchSupport)
+        registerGestureRecognizer(new QWinNativePanGestureRecognizer);
   #endif
 #else
     registerGestureRecognizer(new QTapAndHoldGestureRecognizer);
@@ -124,7 +119,7 @@ Qt::GestureType QGestureManager::registerGestureRecognizer(QGestureRecognizer *r
     if (type == Qt::CustomGesture) {
         // generate a new custom gesture id
         ++m_lastCustomGestureId;
-        type = Qt::GestureType(Qt::CustomGesture + m_lastCustomGestureId);
+        type = Qt::GestureType(m_lastCustomGestureId);
     }
     m_recognizers.insertMulti(type, recognizer);
     delete dummy;
@@ -134,25 +129,31 @@ Qt::GestureType QGestureManager::registerGestureRecognizer(QGestureRecognizer *r
 void QGestureManager::unregisterGestureRecognizer(Qt::GestureType type)
 {
     QList<QGestureRecognizer *> list = m_recognizers.values(type);
-    m_recognizers.remove(type);
+    while (QGestureRecognizer *recognizer = m_recognizers.take(type)) {
+        if (!m_obsoleteGestures.contains(recognizer)) {
+            // inserting even an empty QSet will cause the recognizer to be deleted on destruction of the manager
+            m_obsoleteGestures.insert(recognizer, QSet<QGesture *>());
+        }
+    }
     foreach (QGesture *g, m_gestureToRecognizer.keys()) {
         QGestureRecognizer *recognizer = m_gestureToRecognizer.value(g);
         if (list.contains(recognizer)) {
             m_deletedRecognizers.insert(g, recognizer);
-            m_gestureToRecognizer.remove(g);
         }
     }
 
-    foreach (QGestureRecognizer *recognizer, list) {
-        QList<QGesture *> obsoleteGestures;
-        QMap<ObjectGesture, QList<QGesture *> >::Iterator iter = m_objectGestures.begin();
-        while (iter != m_objectGestures.end()) {
-            ObjectGesture objectGesture = iter.key();
-            if (objectGesture.gesture == type)
-                obsoleteGestures << iter.value();
-            ++iter;
+    QMap<ObjectGesture, QList<QGesture *> >::const_iterator iter = m_objectGestures.begin();
+    while (iter != m_objectGestures.end()) {
+        ObjectGesture objectGesture = iter.key();
+        if (objectGesture.gesture == type) {
+            foreach (QGesture *g, iter.value()) {
+                if (QGestureRecognizer *recognizer = m_gestureToRecognizer.value(g)) {
+                    m_gestureToRecognizer.remove(g);
+                    m_obsoleteGestures[recognizer].insert(g);
+                }
+            }
         }
-        m_obsoleteGestures.insert(recognizer, obsoleteGestures);
+        ++iter;
     }
 }
 
@@ -161,8 +162,18 @@ void QGestureManager::cleanupCachedGestures(QObject *target, Qt::GestureType typ
     QMap<ObjectGesture, QList<QGesture *> >::Iterator iter = m_objectGestures.begin();
     while (iter != m_objectGestures.end()) {
         ObjectGesture objectGesture = iter.key();
-        if (objectGesture.gesture == type && target == objectGesture.object.data()) {
-            qDeleteAll(iter.value());
+        if (objectGesture.gesture == type && target == objectGesture.object) {
+            QSet<QGesture *> gestures = iter.value().toSet();
+            for (QHash<QGestureRecognizer *, QSet<QGesture *> >::iterator
+                 it = m_obsoleteGestures.begin(), e = m_obsoleteGestures.end(); it != e; ++it) {
+                it.value() -= gestures;
+            }
+            foreach (QGesture *g, gestures) {
+                m_deletedRecognizers.remove(g);
+                m_gestureToRecognizer.remove(g);
+            }
+
+            qDeleteAll(gestures);
             iter = m_objectGestures.erase(iter);
         } else {
             ++iter;
@@ -173,7 +184,7 @@ void QGestureManager::cleanupCachedGestures(QObject *target, Qt::GestureType typ
 // get or create a QGesture object that will represent the state for a given object, used by the recognizer
 QGesture *QGestureManager::getState(QObject *object, QGestureRecognizer *recognizer, Qt::GestureType type)
 {
-    // if the widget is being deleted we should be carefull and not to
+    // if the widget is being deleted we should be careful not to
     // create a new state, as it will create QWeakPointer which doesnt work
     // from the destructor.
     if (object->isWidgetType()) {
@@ -184,13 +195,14 @@ QGesture *QGestureManager::getState(QObject *object, QGestureRecognizer *recogni
 #ifndef QT_NO_GRAPHICSVIEW
     } else {
         Q_ASSERT(qobject_cast<QGraphicsObject *>(object));
+        QGraphicsObject *graphicsObject = static_cast<QGraphicsObject *>(object);
+        if (graphicsObject->QGraphicsItem::d_func()->inDestructor)
+            return 0;
 #endif
     }
 
-    QList<QGesture *> states =
-            m_objectGestures.value(QGestureManager::ObjectGesture(object, type));
     // check if the QGesture for this recognizer has already been created
-    foreach (QGesture *state, states) {
+    foreach (QGesture *state, m_objectGestures.value(QGestureManager::ObjectGesture(object, type))) {
         if (m_gestureToRecognizer.value(state) == recognizer)
             return state;
     }
@@ -215,14 +227,13 @@ QGesture *QGestureManager::getState(QObject *object, QGestureRecognizer *recogni
     return state;
 }
 
-bool QGestureManager::filterEventThroughContexts(const QMultiHash<QObject *,
+bool QGestureManager::filterEventThroughContexts(const QMultiMap<QObject *,
                                                  Qt::GestureType> &contexts,
                                                  QEvent *event)
 {
     QSet<QGesture *> triggeredGestures;
     QSet<QGesture *> finishedGestures;
     QSet<QGesture *> newMaybeGestures;
-    QSet<QGesture *> canceledGestures;
     QSet<QGesture *> notGestures;
 
     // TODO: sort contexts by the gesture type and check if one of the contexts
@@ -231,7 +242,7 @@ bool QGestureManager::filterEventThroughContexts(const QMultiHash<QObject *,
     bool ret = false;
 
     // filter the event through recognizers
-    typedef QHash<QObject *, Qt::GestureType>::const_iterator ContextIterator;
+    typedef QMultiMap<QObject *, Qt::GestureType>::const_iterator ContextIterator;
     for (ContextIterator cit = contexts.begin(), ce = contexts.end(); cit != ce; ++cit) {
         Qt::GestureType gestureType = cit.value();
         QMap<Qt::GestureType, QGestureRecognizer *>::const_iterator
@@ -271,6 +282,9 @@ bool QGestureManager::filterEventThroughContexts(const QMultiHash<QObject *,
             }
         }
     }
+    if (triggeredGestures.isEmpty() && finishedGestures.isEmpty()
+        && newMaybeGestures.isEmpty() && notGestures.isEmpty())
+        return ret;
 
     QSet<QGesture *> startedGestures = triggeredGestures - m_activeGestures;
     triggeredGestures &= m_activeGestures;
@@ -278,29 +292,21 @@ bool QGestureManager::filterEventThroughContexts(const QMultiHash<QObject *,
     // check if a running gesture switched back to maybe state
     QSet<QGesture *> activeToMaybeGestures = m_activeGestures & newMaybeGestures;
 
+    // check if a maybe gesture switched to canceled - reset it but don't send an event
+    QSet<QGesture *> maybeToCanceledGestures = m_maybeGestures & notGestures;
+
     // check if a running gesture switched back to not gesture state,
     // i.e. were canceled
-    QSet<QGesture *> activeToCancelGestures = m_activeGestures & notGestures;
-    canceledGestures += activeToCancelGestures;
+    QSet<QGesture *> canceledGestures = m_activeGestures & notGestures;
 
-    // start timers for new gestures in maybe state
-    foreach (QGesture *state, newMaybeGestures) {
-        QBasicTimer &timer = m_maybeGestures[state];
-        if (!timer.isActive())
-            timer.start(3000, this);
-    }
-    // kill timers for gestures that were in maybe state
+    // new gestures in maybe state
+    m_maybeGestures += newMaybeGestures;
+
+    // gestures that were in maybe state
     QSet<QGesture *> notMaybeGestures = (startedGestures | triggeredGestures
                                          | finishedGestures | canceledGestures
                                          | notGestures);
-    foreach(QGesture *gesture, notMaybeGestures) {
-        QHash<QGesture *, QBasicTimer>::iterator it =
-                m_maybeGestures.find(gesture);
-        if (it != m_maybeGestures.end()) {
-            it.value().stop();
-            m_maybeGestures.erase(it);
-        }
-    }
+    m_maybeGestures -= notMaybeGestures;
 
     Q_ASSERT((startedGestures & finishedGestures).isEmpty());
     Q_ASSERT((startedGestures & newMaybeGestures).isEmpty());
@@ -344,11 +350,12 @@ bool QGestureManager::filterEventThroughContexts(const QMultiHash<QObject *,
         !finishedGestures.isEmpty() || !canceledGestures.isEmpty()) {
         DEBUG() << "QGestureManager::filterEventThroughContexts:"
                 << "\n\tactiveGestures:" << m_activeGestures
-                << "\n\tmaybeGestures:" << m_maybeGestures.keys()
+                << "\n\tmaybeGestures:" << m_maybeGestures
                 << "\n\tstarted:" << startedGestures
                 << "\n\ttriggered:" << triggeredGestures
                 << "\n\tfinished:" << finishedGestures
-                << "\n\tcanceled:" << canceledGestures;
+                << "\n\tcanceled:" << canceledGestures
+                << "\n\tmaybe-canceled:" << maybeToCanceledGestures;
     }
 
     QSet<QGesture *> undeliveredGestures;
@@ -369,7 +376,7 @@ bool QGestureManager::filterEventThroughContexts(const QMultiHash<QObject *,
 
     // reset gestures that ended
     QSet<QGesture *> endedGestures =
-            finishedGestures + canceledGestures + undeliveredGestures;
+            finishedGestures + canceledGestures + undeliveredGestures + maybeToCanceledGestures;
     foreach (QGesture *gesture, endedGestures) {
         recycle(gesture);
         m_gestureTargets.remove(gesture);
@@ -449,14 +456,14 @@ void QGestureManager::cleanupGesturesForRemovedRecognizer(QGesture *gesture)
 // return true if accepted (consumed)
 bool QGestureManager::filterEvent(QWidget *receiver, QEvent *event)
 {
-    QSet<Qt::GestureType> types;
-    QMultiHash<QObject *, Qt::GestureType> contexts;
+    QMap<Qt::GestureType, int> types;
+    QMultiMap<QObject *, Qt::GestureType> contexts;
     QWidget *w = receiver;
     typedef QMap<Qt::GestureType, Qt::GestureFlags>::const_iterator ContextIterator;
     if (!w->d_func()->gestureContext.isEmpty()) {
         for(ContextIterator it = w->d_func()->gestureContext.begin(),
             e = w->d_func()->gestureContext.end(); it != e; ++it) {
-            types.insert(it.key());
+            types.insert(it.key(), 0);
             contexts.insertMulti(w, it.key());
         }
     }
@@ -468,7 +475,7 @@ bool QGestureManager::filterEvent(QWidget *receiver, QEvent *event)
              e = w->d_func()->gestureContext.end(); it != e; ++it) {
             if (!(it.value() & Qt::DontStartGestureOnChildren)) {
                 if (!types.contains(it.key())) {
-                    types.insert(it.key());
+                    types.insert(it.key(), 0);
                     contexts.insertMulti(w, it.key());
                 }
             }
@@ -477,20 +484,20 @@ bool QGestureManager::filterEvent(QWidget *receiver, QEvent *event)
             break;
         w = w->parentWidget();
     }
-    return filterEventThroughContexts(contexts, event);
+    return contexts.isEmpty() ? false : filterEventThroughContexts(contexts, event);
 }
 
 #ifndef QT_NO_GRAPHICSVIEW
 bool QGestureManager::filterEvent(QGraphicsObject *receiver, QEvent *event)
 {
-    QSet<Qt::GestureType> types;
-    QMultiHash<QObject *, Qt::GestureType> contexts;
+    QMap<Qt::GestureType, int> types;
+    QMultiMap<QObject *, Qt::GestureType> contexts;
     QGraphicsObject *item = receiver;
     if (!item->QGraphicsItem::d_func()->gestureContext.isEmpty()) {
         typedef QMap<Qt::GestureType, Qt::GestureFlags>::const_iterator ContextIterator;
         for(ContextIterator it = item->QGraphicsItem::d_func()->gestureContext.begin(),
             e = item->QGraphicsItem::d_func()->gestureContext.end(); it != e; ++it) {
-            types.insert(it.key());
+            types.insert(it.key(), 0);
             contexts.insertMulti(item, it.key());
         }
     }
@@ -503,20 +510,23 @@ bool QGestureManager::filterEvent(QGraphicsObject *receiver, QEvent *event)
              e = item->QGraphicsItem::d_func()->gestureContext.end(); it != e; ++it) {
             if (!(it.value() & Qt::DontStartGestureOnChildren)) {
                 if (!types.contains(it.key())) {
-                    types.insert(it.key());
+                    types.insert(it.key(), 0);
                     contexts.insertMulti(item, it.key());
                 }
             }
         }
         item = item->parentObject();
     }
-    return filterEventThroughContexts(contexts, event);
+    return contexts.isEmpty() ? false : filterEventThroughContexts(contexts, event);
 }
 #endif
 
-bool QGestureManager::filterEvent(QGesture *state, QEvent *event)
+bool QGestureManager::filterEvent(QObject *receiver, QEvent *event)
 {
-    QMultiHash<QObject *, Qt::GestureType> contexts;
+    if (!m_gestureToRecognizer.contains(static_cast<QGesture *>(receiver)))
+        return false;
+    QGesture *state = static_cast<QGesture *>(receiver);
+    QMultiMap<QObject *, Qt::GestureType> contexts;
     contexts.insert(state, state->gestureType());
     return filterEventThroughContexts(contexts, event);
 }
@@ -586,8 +596,9 @@ void QGestureManager::deliverEvents(const QSet<QGesture *> &gestures,
             if (gesture->hasHotSpot()) {
                 // guess the target widget using the hotspot of the gesture
                 QPoint pt = gesture->hotSpot().toPoint();
-                if (QWidget *w = qApp->topLevelAt(pt)) {
-                    target = w->childAt(w->mapFromGlobal(pt));
+                if (QWidget *topLevel = qApp->topLevelAt(pt)) {
+                    QWidget *child = topLevel->childAt(topLevel->mapFromGlobal(pt));
+                    target = child ? child : topLevel;
                 }
             } else {
                 // or use the context of the gesture
@@ -601,6 +612,7 @@ void QGestureManager::deliverEvents(const QSet<QGesture *> &gestures,
 
         Qt::GestureType gestureType = gesture->gestureType();
         Q_ASSERT(gestureType != Qt::CustomGesture);
+        Q_UNUSED(gestureType);
 
         if (target) {
             if (gesture->state() == Qt::GestureStarted) {
@@ -679,32 +691,13 @@ void QGestureManager::deliverEvents(const QSet<QGesture *> &gestures,
     }
 }
 
-void QGestureManager::timerEvent(QTimerEvent *event)
-{
-    QHash<QGesture *, QBasicTimer>::iterator it = m_maybeGestures.begin(),
-                                             e = m_maybeGestures.end();
-    for (; it != e; ) {
-        QBasicTimer &timer = it.value();
-        Q_ASSERT(timer.isActive());
-        if (timer.timerId() == event->timerId()) {
-            timer.stop();
-            QGesture *gesture = it.key();
-            it = m_maybeGestures.erase(it);
-            DEBUG() << "QGestureManager::timerEvent: gesture stopped due to timeout:"
-                    << gesture;
-            recycle(gesture);
-        } else {
-            ++it;
-        }
-    }
-}
-
 void QGestureManager::recycle(QGesture *gesture)
 {
     QGestureRecognizer *recognizer = m_gestureToRecognizer.value(gesture, 0);
     if (recognizer) {
         gesture->setGestureCancelPolicy(QGesture::CancelNone);
         recognizer->reset(gesture);
+        m_activeGestures.remove(gesture);
     } else {
         cleanupGesturesForRemovedRecognizer(gesture);
     }
@@ -712,4 +705,6 @@ void QGestureManager::recycle(QGesture *gesture)
 
 QT_END_NAMESPACE
 
+#endif // QT_NO_GESTURES
+
 #include "moc_qgesturemanager_p.cpp"
