@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -48,6 +48,7 @@
 #include "qline.h"
 #include "qdebug.h"
 #include <QtCore/qcoreapplication.h>
+#include "private/qstylehelper_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -96,9 +97,11 @@ const uchar *qt_patternForBrush(int brushStyle, bool invert)
 
 QPixmap qt_pixmapForBrush(int brushStyle, bool invert)
 {
+
     QPixmap pm;
-    QString key = QLatin1String("$qt-brush$") + QString::number(brushStyle)
-                  + QString::number((int)invert);
+    QString key = QLatin1Literal("$qt-brush$")
+                  % HexString<uint>(brushStyle)
+                  % QLatin1Char(invert ? '1' : '0');
     if (!QPixmapCache::find(key, pm)) {
         pm = QBitmap::fromData(QSize(8, 8), qt_patternForBrush(brushStyle, invert),
                                QImage::Format_MonoLSB);
@@ -160,8 +163,7 @@ static void qt_cleanup_brush_pattern_image_cache()
     qt_brushPatternImageCache()->cleanup();
 }
 
-Q_GUI_EXPORT
-QImage qt_imageForBrush(int brushStyle, bool invert)
+Q_GUI_EXPORT QImage qt_imageForBrush(int brushStyle, bool invert)
 {
     return qt_brushPatternImageCache()->getImage(brushStyle, invert);
 }
@@ -330,8 +332,8 @@ struct QBrushDataPointerDeleter
 
     \endtable
 
-    For more information about painting in general, see \l{The Paint
-    System} documentation.
+    For more information about painting in general, see the \l{Paint
+    System}.
 
     \sa Qt::BrushStyle, QPainter, QColor
 */
@@ -912,7 +914,7 @@ void QBrush::setTransform(const QTransform &matrix)
     otherwise returns false.
 
     Two brushes are different if they have different styles, colors or
-    pixmaps.
+    transforms or different pixmaps or gradients depending on the style.
 
     \sa operator==()
 */
@@ -924,7 +926,7 @@ void QBrush::setTransform(const QTransform &matrix)
     otherwise returns false.
 
     Two brushes are equal if they have equal styles, colors and
-    pixmaps.
+    transforms and equal pixmaps or gradients depending on the style.
 
     \sa operator!=()
 */
@@ -933,26 +935,26 @@ bool QBrush::operator==(const QBrush &b) const
 {
     if (b.d == d)
         return true;
-    if (b.d->style == d->style && b.d->color == d->color) {
-        switch (d->style) {
-        case Qt::TexturePattern: {
-            QPixmap &us = (static_cast<QTexturedBrushData *>(d.data()))->pixmap();
-            QPixmap &them = (static_cast<QTexturedBrushData *>(b.d.data()))->pixmap();
+    if (b.d->style != d->style || b.d->color != d->color || b.d->transform != d->transform)
+        return false;
+    switch (d->style) {
+    case Qt::TexturePattern:
+        {
+            const QPixmap &us = (static_cast<QTexturedBrushData *>(d.data()))->pixmap();
+            const QPixmap &them = (static_cast<QTexturedBrushData *>(b.d.data()))->pixmap();
             return ((us.isNull() && them.isNull()) || us.cacheKey() == them.cacheKey());
         }
-        case Qt::LinearGradientPattern:
-        case Qt::RadialGradientPattern:
-        case Qt::ConicalGradientPattern:
-            {
-                QGradientBrushData *d1 = static_cast<QGradientBrushData *>(d.data());
-                QGradientBrushData *d2 = static_cast<QGradientBrushData *>(b.d.data());
-                return d1->gradient == d2->gradient;
-            }
-        default:
-            return true;
+    case Qt::LinearGradientPattern:
+    case Qt::RadialGradientPattern:
+    case Qt::ConicalGradientPattern:
+        {
+            const QGradientBrushData *d1 = static_cast<QGradientBrushData *>(d.data());
+            const QGradientBrushData *d2 = static_cast<QGradientBrushData *>(b.d.data());
+            return d1->gradient == d2->gradient;
         }
+    default:
+        return true;
     }
-    return false;
 }
 
 /*!
@@ -970,7 +972,7 @@ bool QBrush::operator==(const QBrush &b) const
 QDebug operator<<(QDebug dbg, const QBrush &b)
 {
 #ifndef Q_BROKEN_DEBUG_STREAM
-    const char *BRUSH_STYLES[] = {
+    static const char *BRUSH_STYLES[] = {
      "NoBrush",
      "SolidPattern",
      "Dense1Pattern",
@@ -989,7 +991,8 @@ QDebug operator<<(QDebug dbg, const QBrush &b)
      "LinearGradientPattern",
      "RadialGradientPattern",
      "ConicalGradientPattern",
-     "TexturePattern"
+     0, 0, 0, 0, 0, 0,
+     "TexturePattern" // 24
     };
 
     dbg.nospace() << "QBrush(" << b.color() << ',' << BRUSH_STYLES[b.style()] << ')';
@@ -1013,7 +1016,7 @@ QDebug operator<<(QDebug dbg, const QBrush &b)
     Writes the given \a brush to the given \a stream and returns a
     reference to the \a stream.
 
-    \sa {Format of the QDataStream Operators}
+    \sa {Serializing Qt Data Types}
 */
 
 QDataStream &operator<<(QDataStream &s, const QBrush &b)
@@ -1081,7 +1084,7 @@ QDataStream &operator<<(QDataStream &s, const QBrush &b)
     Reads the given \a brush from the given \a stream and returns a
     reference to the \a stream.
 
-    \sa {Format of the QDataStream Operators}
+    \sa {Serializing Qt Data Types}
 */
 
 QDataStream &operator>>(QDataStream &s, QBrush &b)
@@ -1215,17 +1218,16 @@ QDataStream &operator>>(QDataStream &s, QBrush &b)
     \o QConicalGradient
     \endtable
 
-    The colors in a gradient is defined using stop points of the
-    QGradientStop type, i.e. a position and a color.  Use the
-    setColorAt() function to define a single stop
-    point. Alternatively, use the setStops() function to define
-    several stop points in one go. Note that the latter function \e
-    replaces the current set of stop points.
+    The colors in a gradient are defined using stop points of the
+    QGradientStop type; i.e., a position and a color. Use the setColorAt()
+    function to define a single stop point. Alternatively, use the
+    setStops() function to define several stop points in one go. Note that
+    the latter function \e replaces the current set of stop points.
 
     It is the gradient's complete set of stop points (accessible
     through the stops() function) that describes how the gradient area
-    should be filled. If no stop points have been specified, a
-    gradient of black at 0 to white at 1 is used.
+    should be filled. If no stop points have been specified, a gradient
+    of black at 0 to white at 1 is used.
 
     A diagonal linear gradient from black at (100, 100) to white at
     (200, 200) could be specified like this:
