@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -64,7 +64,8 @@ enum {
 
 QTextCursorPrivate::QTextCursorPrivate(QTextDocumentPrivate *p)
     : priv(p), x(0), position(0), anchor(0), adjusted_anchor(0),
-      currentCharFormat(-1), visualNavigation(false)
+      currentCharFormat(-1), visualNavigation(false), keepPositionOnInsert(false),
+      changed(false)
 {
     priv->addCursor(this);
 }
@@ -79,6 +80,8 @@ QTextCursorPrivate::QTextCursorPrivate(const QTextCursorPrivate &rhs)
     x = rhs.x;
     currentCharFormat = rhs.currentCharFormat;
     visualNavigation = rhs.visualNavigation;
+    keepPositionOnInsert = rhs.keepPositionOnInsert;
+    changed = rhs.changed;
     priv->addCursor(this);
 }
 
@@ -95,7 +98,7 @@ QTextCursorPrivate::AdjustResult QTextCursorPrivate::adjustPosition(int position
     if (position < positionOfChange
         || (position == positionOfChange
             && (op == QTextUndoCommand::KeepCursor
-                || anchor < position)
+                || keepPositionOnInsert)
             )
          ) {
         result = CursorUnchanged;
@@ -361,7 +364,7 @@ bool QTextCursorPrivate::movePosition(QTextCursor::MoveOperation op, QTextCursor
     QTextBlock blockIt = block();
 
     if (op >= QTextCursor::Left && op <= QTextCursor::WordRight
-        && blockIt.blockFormat().layoutDirection() == Qt::RightToLeft) {
+        && blockIt.textDirection() == Qt::RightToLeft) {
         if (op == QTextCursor::Left)
             op = QTextCursor::NextCharacter;
         else if (op == QTextCursor::Right)
@@ -862,27 +865,28 @@ QTextLayout *QTextCursorPrivate::blockLayout(QTextBlock &block) const{
     \ingroup richtext-processing
     \ingroup shared
 
-
-    Text cursors are objects that are used to access and modify the contents
-    and underlying structure of text documents via a programming interface
-    that mimics the behavior of a cursor in a text editor. QTextCursor contains
-    information about both the cursor's position within a QTextDocument and any
-    selection that it has made.
+    Text cursors are objects that are used to access and modify the
+    contents and underlying structure of text documents via a
+    programming interface that mimics the behavior of a cursor in a
+    text editor. QTextCursor contains information about both the
+    cursor's position within a QTextDocument and any selection that it
+    has made.
 
     QTextCursor is modeled on the way a text cursor behaves in a text
-    editor, providing a programmatic means of performing standard actions
-    through the user interface. A document can be thought of as a
-    single string of characters with the cursor's position() being \e
-    between any two characters (or at the very beginning or very end
-    of the document). Documents can also contain tables, lists,
-    images, and other objects in addition to text but, from the developer's
-    point of view, the document can be treated as one long string.
-    Some portions of that string can be considered to lie within particular
-    blocks (e.g. paragraphs), or within a table's cell, or a list's item,
-    or other structural elements. When we refer to "current character" we
-    mean the character immediately after the cursor position() in the
-    document; similarly the "current block" is the block that contains the
-    cursor position().
+    editor, providing a programmatic means of performing standard
+    actions through the user interface. A document can be thought of
+    as a single string of characters. The cursor's current position()
+    then is always either \e between two consecutive characters in the
+    string, or else \e before the very first character or \e after the
+    very last character in the string.  Documents can also contain
+    tables, lists, images, and other objects in addition to text but,
+    from the developer's point of view, the document can be treated as
+    one long string.  Some portions of that string can be considered
+    to lie within particular blocks (e.g. paragraphs), or within a
+    table's cell, or a list's item, or other structural elements. When
+    we refer to "current character" we mean the character immediately
+    \e before the cursor position() in the document. Similarly, the
+    "current block" is the block that contains the cursor position().
 
     A QTextCursor also has an anchor() position. The text that is
     between the anchor() and the position() is the selection. If
@@ -940,11 +944,12 @@ QTextLayout *QTextCursorPrivate::blockLayout(QTextBlock &block) const{
     undo/redo) using beginEditBlock() and endEditBlock().
 
     Cursor movements are limited to valid cursor positions. In Latin
-    writing this is usually after every character in the text. In some
-    other writing systems cursor movements are limited to "clusters"
-    (e.g. a syllable in Devanagari, or a base letter plus diacritics).
-    Functions such as movePosition() and deleteChar() limit cursor
-    movement to these valid positions.
+    writing this is between any two consecutive characters in the
+    text, before the first character, or after the last character. In
+    some other writing systems cursor movements are limited to
+    "clusters" (e.g. a syllable in Devanagari, or a base letter plus
+    diacritics).  Functions such as movePosition() and deleteChar()
+    limit cursor movement to these valid positions.
 
     \sa \link richtext.html Rich Text Processing\endlink
 
@@ -1143,7 +1148,7 @@ void QTextCursor::setPosition(int pos, MoveMode m)
     Returns the absolute position of the cursor within the document.
     The cursor is positioned between characters.
 
-    \sa setPosition() movePosition() anchor()
+    \sa setPosition() movePosition() anchor() positionInBlock()
 */
 int QTextCursor::position() const
 {
@@ -1152,6 +1157,22 @@ int QTextCursor::position() const
     return d->position;
 }
 
+/*!
+    \since 4.7
+    Returns the relative position of the cursor within the block.
+    The cursor is positioned between characters.
+
+    This is equivalent to \c{ position() - block().position()}.
+
+    \sa position()
+*/
+int QTextCursor::positionInBlock() const
+{
+    if (!d || !d->priv)
+        return 0;
+    return d->position - d->block().position();
+}
+
 /*!
     Returns the anchor position; this is the same as position() unless
     there is a selection in which case position() marks one end of the
@@ -1258,6 +1279,80 @@ void QTextCursor::setVisualNavigation(bool b)
         d->visualNavigation = b;
 }
 
+
+/*!
+  \since 4.7
+
+  Sets the visual x position for vertical cursor movements to \a x.
+
+  The vertical movement x position is cleared automatically when the cursor moves horizontally, and kept
+  unchanged when the cursor moves vertically. The mechanism allows the cursor to move up and down on a
+  visually straight line with proportional fonts, and to gently "jump" over short lines.
+
+  A value of -1 indicates no predefined x position. It will then be set automatically the next time the
+  cursor moves up or down.
+
+  \sa verticalMovementX()
+  */
+void QTextCursor::setVerticalMovementX(int x)
+{
+    if (d)
+        d->x = x;
+}
+
+/*! \since 4.7
+
+  Returns the visual x position for vertical cursor movements.
+
+  A value of -1 indicates no predefined x position. It will then be set automatically the next time the
+  cursor moves up or down.
+
+  \sa setVerticalMovementX()
+  */
+int QTextCursor::verticalMovementX() const
+{
+    return d ? d->x : -1;
+}
+
+/*!
+  \since 4.7
+
+  Returns whether the cursor should keep its current position when text gets inserted at the position of the
+  cursor.
+
+  The default is false;
+
+  \sa setKeepPositionOnInsert()
+ */
+bool QTextCursor::keepPositionOnInsert() const
+{
+    return d ? d->keepPositionOnInsert : false;
+}
+
+/*!
+  \since 4.7
+
+  Defines whether the cursor should keep its current position when text gets inserted at the current position of the
+  cursor.
+
+  If \a b is true, the cursor keeps its current position when text gets inserted at the positing of the cursor.
+  If \a b is false, the cursor moves along with the inserted text.
+
+  The default is false.
+
+  Note that a cursor always moves when text is inserted before the current position of the cursor, and it
+  always keeps its position when text is inserted after the current position of the cursor.
+
+  \sa keepPositionOnInsert()
+ */
+void QTextCursor::setKeepPositionOnInsert(bool b)
+{
+    if (d)
+        d->keepPositionOnInsert = b;
+}
+
+
+
 /*!
     Inserts \a text at the current position, using the current
     character format.
@@ -1390,16 +1485,16 @@ void QTextCursor::deletePreviousChar()
 {
     if (!d || !d->priv)
         return;
-    
+
     if (d->position != d->anchor) {
         removeSelectedText();
         return;
     }
-    
+
     if (d->anchor < 1 || !d->canDelete(d->anchor-1))
         return;
     d->anchor--;
-    
+
     QTextDocumentPrivate::FragmentIterator fragIt = d->priv->find(d->anchor);
     const QTextFragmentData * const frag = fragIt.value();
     int fpos = fragIt.position();
@@ -1411,7 +1506,7 @@ void QTextCursor::deletePreviousChar()
         if (uc.unicode() >= 0xd800 && uc.unicode() < 0xdc00)
             --d->anchor;
     }
-    
+
     d->adjusted_anchor = d->anchor;
     d->remove();
     d->setX();
@@ -1739,8 +1834,9 @@ void QTextCursor::mergeBlockCharFormat(const QTextCharFormat &modifier)
 }
 
 /*!
-    Returns the format of the character immediately before the cursor position(). If the cursor is
-    positioned at the beginning of a text block that is not empty then the format of the character
+    Returns the format of the character immediately before the cursor
+    position(). If the cursor is positioned at the beginning of a text
+    block that is not empty then the format of the character
     immediately after the cursor is returned.
 
     \sa insertText(), blockFormat()
@@ -2343,6 +2439,9 @@ void QTextCursor::beginEditBlock()
     if (!d || !d->priv)
         return;
 
+    if (d->priv->editBlock == 0) // we are the initial edit block, store current cursor position for undo
+        d->priv->editBlockCursorPosition = d->position;
+
     d->priv->beginEditBlock();
 }
 
@@ -2411,9 +2510,17 @@ int QTextCursor::blockNumber() const
     return d->block().blockNumber();
 }
 
+
 /*!
     \since 4.2
     Returns the position of the cursor within its containing line.
+
+    Note that this is the column number relative to a wrapped line,
+    not relative to the block (i.e. the paragraph).
+
+    You probably want to call positionInBlock() instead.
+
+    \sa positionInBlock()
 */
 int QTextCursor::columnNumber() const
 {
