@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -63,7 +63,7 @@ QT_BEGIN_NAMESPACE
 // The VxWorks DIAB compiler crashes when initializing the anonymouse union with { a7 }
 #if !defined(Q_CC_DIAB)
 #  define QT_INIT_TEXTUNDOCOMMAND(c, a1, a2, a3, a4, a5, a6, a7, a8) \
-          QTextUndoCommand c = { a1, a2, 0, 0, a3, a4, a5, a6, { a7 }, a8 }
+          QTextUndoCommand c = { a1, a2, 0, 0, quint8(a3), a4, a5, a6, { a7 }, a8 }
 #else
 #  define QT_INIT_TEXTUNDOCOMMAND(c, a1, a2, a3, a4, a5, a6, a7, a8) \
           QTextUndoCommand c = { a1, a2, 0, 0, a3, a4, a5, a6 }; c.blockFormat = a7; c.revision = a8
@@ -192,6 +192,7 @@ QTextDocumentPrivate::QTextDocumentPrivate()
     initialBlockCharFormatIndex(-1) // set correctly later in init()
 {
     editBlock = 0;
+    editBlockCursorPosition = -1;
     docChangeFrom = -1;
 
     undoState = 0;
@@ -204,6 +205,7 @@ QTextDocumentPrivate::QTextDocumentPrivate()
 
     undoEnabled = true;
     inContentsChange = false;
+    blockCursorAdjustment = false;
 
     defaultTextOption.setTabStop(80); // same as in qtextengine.cpp
     defaultTextOption.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
@@ -233,17 +235,17 @@ void QTextDocumentPrivate::init()
 void QTextDocumentPrivate::clear()
 {
     Q_Q(QTextDocument);
-    for (int i = 0; i < cursors.count(); ++i) {
-        cursors.at(i)->setPosition(0);
-        cursors.at(i)->currentCharFormat = -1;
-        cursors.at(i)->anchor = 0;
-        cursors.at(i)->adjusted_anchor = 0;
+
+    foreach (QTextCursorPrivate *curs, cursors) {
+        curs->setPosition(0);
+        curs->currentCharFormat = -1;
+        curs->anchor = 0;
+        curs->adjusted_anchor = 0;
     }
 
     QList<QTextCursorPrivate *>oldCursors = cursors;
     QT_TRY{
         cursors.clear();
-        changedCursors.clear();
 
         QMap<int, QTextObject *>::Iterator objectIt = objects.begin();
         while (objectIt != objects.end()) {
@@ -259,8 +261,7 @@ void QTextDocumentPrivate::clear()
         objects.clear();
 
         title.clear();
-        undoState = 0;
-        truncateUndoStack();
+        clearUndoRedoStacks(QTextDocument::UndoAndRedoStacks);
         text = QString();
         unreachableCharacterCount = 0;
         modifiedState = 0;
@@ -287,12 +288,12 @@ void QTextDocumentPrivate::clear()
 
 QTextDocumentPrivate::~QTextDocumentPrivate()
 {
-    for (int i = 0; i < cursors.count(); ++i)
-        cursors.at(i)->priv = 0;
+    foreach (QTextCursorPrivate *curs, cursors)
+        curs->priv = 0;
     cursors.clear();
     undoState = 0;
     undoEnabled = true;
-    truncateUndoStack();
+    clearUndoRedoStacks(QTextDocument::RedoStack);
 }
 
 void QTextDocumentPrivate::setLayout(QAbstractTextDocumentLayout *layout)
@@ -669,7 +670,14 @@ void QTextDocumentPrivate::remove(int pos, int length, QTextUndoCommand::Operati
 {
     if (length == 0)
         return;
+    blockCursorAdjustment = true;
     move(pos, -1, length, op);
+    blockCursorAdjustment = false;
+    foreach (QTextCursorPrivate *curs, cursors) {
+        if (curs->adjustPosition(pos, -length, op) == QTextCursorPrivate::CursorMoved) {
+            curs->changed = true;
+        }
+    }
 }
 
 void QTextDocumentPrivate::setCharFormat(int pos, int length, const QTextCharFormat &newFormat, FormatChangeMode mode)
@@ -870,6 +878,8 @@ int QTextDocumentPrivate::undoRedo(bool undo)
 
     undoEnabled = false;
     beginEditBlock();
+    int editPos = -1;
+    int editLength = -1;
     while (1) {
         if (undo)
             --undoState;
@@ -881,11 +891,17 @@ int QTextDocumentPrivate::undoRedo(bool undo)
             remove(c.pos, c.length, (QTextUndoCommand::Operation)c.operation);
             PMDEBUG("   erase: from %d, length %d", c.pos, c.length);
             c.command = QTextUndoCommand::Removed;
+            editPos = c.pos;
+            editLength = 0;
 	    break;
         case QTextUndoCommand::Removed:
             PMDEBUG("   insert: format %d (from %d, length %d, strpos=%d)", c.format, c.pos, c.length, c.strPos);
             insert_string(c.pos, c.strPos, c.length, c.format, (QTextUndoCommand::Operation)c.operation);
             c.command = QTextUndoCommand::Inserted;
+            if (editPos != (int)c.pos)
+                editLength = 0;
+            editPos = c.pos;
+            editLength += c.length;
 	    break;
 	case QTextUndoCommand::BlockInserted:
 	case QTextUndoCommand::BlockAdded:
@@ -895,6 +911,8 @@ int QTextDocumentPrivate::undoRedo(bool undo)
 		c.command = QTextUndoCommand::BlockRemoved;
 	    else
 		c.command = QTextUndoCommand::BlockDeleted;
+            editPos = c.pos;
+            editLength = 0;
 	    break;
 	case QTextUndoCommand::BlockRemoved:
 	case QTextUndoCommand::BlockDeleted:
@@ -905,6 +923,10 @@ int QTextDocumentPrivate::undoRedo(bool undo)
 		c.command = QTextUndoCommand::BlockInserted;
 	    else
 		c.command = QTextUndoCommand::BlockAdded;
+            if (editPos != (int)c.pos)
+                editLength = 0;
+            editPos = c.pos;
+            editLength += 1;
 	    break;
 	case QTextUndoCommand::CharFormatChanged: {
             resetBlockRevision = -1; // ## TODO
@@ -915,6 +937,10 @@ int QTextDocumentPrivate::undoRedo(bool undo)
             int oldFormat = it.value()->format;
             setCharFormat(c.pos, c.length, formats.charFormat(c.format));
             c.format = oldFormat;
+            if (editPos != (int)c.pos)
+                editLength = 0;
+            editPos = c.pos;
+            editLength += c.length;
 	    break;
 	}
 	case QTextUndoCommand::BlockFormatChanged: {
@@ -937,6 +963,7 @@ int QTextDocumentPrivate::undoRedo(bool undo)
                 group->blockFormatChanged(it);
             }
             documentChange(it.position(), it.length());
+            editPos = -1;
 	    break;
 	}
 	case QTextUndoCommand::GroupFormatChange: {
@@ -946,14 +973,20 @@ int QTextDocumentPrivate::undoRedo(bool undo)
             int oldFormat = formats.objectFormatIndex(c.objectIndex);
             changeObjectFormat(object, c.format);
             c.format = oldFormat;
+            editPos = -1;
 	    break;
 	}
+        case QTextUndoCommand::CursorMoved:
+            editPos = c.pos;
+            editLength = 0;
+            break;
 	case QTextUndoCommand::Custom:
             resetBlockRevision = -1; // ## TODO
             if (undo)
                 c.custom->undo();
             else
                 c.custom->redo();
+            editPos = -1;
 	    break;
 	default:
 	    Q_ASSERT(false);
@@ -979,14 +1012,19 @@ int QTextDocumentPrivate::undoRedo(bool undo)
             break;
     }
     undoEnabled = true;
-    int editPos = -1;
-    if (docChangeFrom >= 0) {
-        editPos = qMin(docChangeFrom + docChangeLength, length() - 1);
-    }
+
+    int newCursorPos = -1;
+
+    if (editPos >=0)
+        newCursorPos = editPos + editLength;
+    else if (docChangeFrom >= 0)
+        newCursorPos= qMin(docChangeFrom + docChangeLength, length() - 1);
+
     endEditBlock();
     emitUndoAvailable(isUndoAvailable());
     emitRedoAvailable(isRedoAvailable());
-    return editPos;
+
+    return newCursorPos;
 }
 
 /*!
@@ -1019,7 +1057,19 @@ void QTextDocumentPrivate::appendUndoItem(const QTextUndoCommand &c)
     if (!undoEnabled)
         return;
     if (undoState < undoStack.size())
-        truncateUndoStack();
+        clearUndoRedoStacks(QTextDocument::RedoStack);
+
+    if (editBlock != 0 && editBlockCursorPosition >= 0) { // we had a beginEditBlock() with a cursor position
+        if (c.pos != (quint32) editBlockCursorPosition) { // and that cursor position is different from the command
+            // generate a CursorMoved undo item
+            QT_INIT_TEXTUNDOCOMMAND(cc, QTextUndoCommand::CursorMoved, true, QTextUndoCommand::MoveCursor,
+                                    0, 0, editBlockCursorPosition, 0, 0);
+            undoStack.append(cc);
+            undoState++;
+            editBlockCursorPosition = -1;
+        }
+    }
+
 
     if (!undoStack.isEmpty() && modified) {
         QTextUndoCommand &last = undoStack[undoState - 1];
@@ -1042,26 +1092,46 @@ void QTextDocumentPrivate::appendUndoItem(const QTextUndoCommand &c)
         emit document()->undoCommandAdded();
 }
 
-void QTextDocumentPrivate::truncateUndoStack()
+void QTextDocumentPrivate::clearUndoRedoStacks(QTextDocument::Stacks stacksToClear,
+                                               bool emitSignals)
 {
-    if (undoState == undoStack.size())
-        return;
-
-    for (int i = undoState; i < undoStack.size(); ++i) {
-        QTextUndoCommand c = undoStack[i];
-        if (c.command & QTextUndoCommand::Removed) {
-            // ########
-//             QTextFragment *f = c.fragment_list;
-//             while (f) {
-//                 QTextFragment *n = f->right;
-//                 delete f;
-//                 f = n;
-//             }
-        } else if (c.command & QTextUndoCommand::Custom) {
-            delete c.custom;
+    bool undoCommandsAvailable = undoState != 0;
+    bool redoCommandsAvailable = undoState != undoStack.size();
+    if (stacksToClear == QTextDocument::UndoStack && undoCommandsAvailable) {
+        for (int i = 0; i < undoState; ++i) {
+            QTextUndoCommand c = undoStack[undoState];
+            if (c.command & QTextUndoCommand::Custom)
+                delete c.custom;
         }
+        undoStack.remove(0, undoState);
+        undoStack.resize(undoStack.size() - undoState);
+        undoState = 0;
+        if (emitSignals)
+            emitUndoAvailable(false);
+    } else if (stacksToClear == QTextDocument::RedoStack
+               && redoCommandsAvailable) {
+        for (int i = undoState; i < undoStack.size(); ++i) {
+            QTextUndoCommand c = undoStack[i];
+            if (c.command & QTextUndoCommand::Custom)
+                delete c.custom;
+        }
+        undoStack.resize(undoState);
+        if (emitSignals)
+            emitRedoAvailable(false);
+    } else if (stacksToClear == QTextDocument::UndoAndRedoStacks
+               && !undoStack.isEmpty()) {
+        for (int i = 0; i < undoStack.size(); ++i) {
+            QTextUndoCommand c = undoStack[i];
+            if (c.command & QTextUndoCommand::Custom)
+                delete c.custom;
+        }
+        undoState = 0;
+        undoStack.resize(0);
+        if (emitSignals && undoCommandsAvailable)
+            emitUndoAvailable(false);
+        if (emitSignals && redoCommandsAvailable)
+            emitRedoAvailable(false);
     }
-    undoStack.resize(undoState);
 }
 
 void QTextDocumentPrivate::emitUndoAvailable(bool available)
@@ -1089,7 +1159,7 @@ void QTextDocumentPrivate::enableUndoRedo(bool enable)
 
     if (!enable) {
         undoState = 0;
-        truncateUndoStack();
+        clearUndoRedoStacks(QTextDocument::RedoStack);
         emitUndoAvailable(false);
         emitRedoAvailable(false);
     }
@@ -1122,6 +1192,8 @@ void QTextDocumentPrivate::endEditBlock()
         }
     }
 
+    editBlockCursorPosition = -1;
+
     finishEdit();
 }
 
@@ -1157,10 +1229,15 @@ void QTextDocumentPrivate::finishEdit()
         }
     }
 
-    while (!changedCursors.isEmpty()) {
-        QTextCursorPrivate *curs = changedCursors.takeFirst();
-        emit q->cursorPositionChanged(QTextCursor(curs));
+    QList<QTextCursor> changedCursors;
+    foreach (QTextCursorPrivate *curs, cursors) {
+        if (curs->changed) {
+            curs->changed = false;
+            changedCursors.append(QTextCursor(curs));
+        }
     }
+    foreach (const QTextCursor &cursor, changedCursors)
+        emit q->cursorPositionChanged(cursor);
 
     contentsChanged();
 
@@ -1202,11 +1279,13 @@ void QTextDocumentPrivate::adjustDocumentChangesAndCursors(int from, int addedOr
     if (!editBlock)
         ++revision;
 
-    for (int i = 0; i < cursors.size(); ++i) {
-        QTextCursorPrivate *curs = cursors.at(i);
-        if (curs->adjustPosition(from, addedOrRemoved, op) == QTextCursorPrivate::CursorMoved) {
-            if (!changedCursors.contains(curs))
-                changedCursors.append(curs);
+    if (blockCursorAdjustment)  {
+        ; // postpone, will be called again from QTextDocumentPrivate::remove()
+    } else {
+        foreach (QTextCursorPrivate *curs, cursors) {
+            if (curs->adjustPosition(from, addedOrRemoved, op) == QTextCursorPrivate::CursorMoved) {
+                curs->changed = true;
+            }
         }
     }
 
@@ -1570,7 +1649,7 @@ void QTextDocumentPrivate::compressPieceTable()
     int newLen = 0;
 
     for (FragmentMap::Iterator it = fragments.begin(); !it.atEnd(); ++it) {
-        qMemCopy(newTextPtr, text.constData() + it->stringPosition, it->size_array[0] * sizeof(QChar));
+        memcpy(newTextPtr, text.constData() + it->stringPosition, it->size_array[0] * sizeof(QChar));
         it->stringPosition = newLen;
         newTextPtr += it->size_array[0];
         newLen += it->size_array[0];
@@ -1629,8 +1708,8 @@ bool QTextDocumentPrivate::ensureMaximumBlockCount()
 void QTextDocumentPrivate::aboutToRemoveCell(int from, int to)
 {
     Q_ASSERT(from <= to);
-    for (int i = 0; i < cursors.size(); ++i)
-        cursors.at(i)->aboutToRemoveCell(from, to);
+    foreach (QTextCursorPrivate *curs, cursors)
+        curs->aboutToRemoveCell(from, to);
 }
 
 QT_END_NAMESPACE
