@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -248,7 +248,7 @@ void QDateTimeEdit::setDateTime(const QDateTime &datetime)
 
 /*!
   \property QDateTimeEdit::date
-  \brief the QDate that is set in the QDateTimeEdit
+  \brief the QDate that is set in the widget
 
   By default, this property contains a date that refers to January 1, 2000.
 
@@ -279,7 +279,7 @@ void QDateTimeEdit::setDate(const QDate &date)
 
 /*!
   \property QDateTimeEdit::time
-  \brief the QTime that is set in the QDateTimeEdit
+  \brief the QTime that is set in the widget
 
   By default, this property contains a time of 00:00:00 and 0 milliseconds.
 
@@ -754,6 +754,7 @@ QCalendarWidget *QDateTimeEdit::calendarWidget() const
   Sets the given \a calendarWidget as the widget to be used for the calendar
   pop-up. The editor does not automatically take ownership of the calendar widget.
 
+  \note calendarPopup must be set to true before setting the calendar widget.
   \sa calendarPopup
 */
 void QDateTimeEdit::setCalendarWidget(QCalendarWidget *calendarWidget)
@@ -1175,7 +1176,7 @@ void QDateTimeEdit::keyPressEvent(QKeyEvent *event)
         return; }
     }
     QAbstractSpinBox::keyPressEvent(event);
-    if (select && !(event->modifiers() & Qt::ShiftModifier) && !d->edit->hasSelectedText()) {
+    if (select && !d->edit->hasSelectedText()) {
         if (inserted && d->sectionAt(d->edit->cursorPosition()) == QDateTimeParser::NoSectionIndex) {
             QString str = d->displayText();
             int pos = d->edit->cursorPosition();
@@ -1554,13 +1555,6 @@ QTimeEdit::QTimeEdit(const QTime &time, QWidget *parent)
 {
 }
 
-/*!
-    \property QTimeEdit::time
-    \brief the QTime that is shown in the widget
-
-    By default, this property contains a time of 00:00:00 and 0 milliseconds.
-*/
-
 
 /*!
   \class QDateEdit
@@ -1615,13 +1609,6 @@ QDateEdit::QDateEdit(const QDate &date, QWidget *parent)
 {
 }
 
-/*!
-    \property QDateEdit::date
-    \brief the QDate that is shown in the widget
-
-    By default, this property contains a date referring to January 1, 2000.
-*/
-
 
 // --- QDateTimeEditPrivate ---
 
