@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -53,6 +53,7 @@
 #include <qurl.h>
 #include "qlabel_p.h"
 #include "private/qstylesheetstyle_p.h"
+#include <qmath.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -634,7 +635,7 @@ QSize QLabelPrivate::sizeForWidth(int w) const
         br = movie->currentPixmap().rect();
 #endif
     else if (isTextLabel) {
-        int align = QStyle::visualAlignment(q->layoutDirection(), QFlag(this->align));
+        int align = QStyle::visualAlignment(textDirection(), QFlag(this->align));
         // Add indentation
         int m = indent;
 
@@ -661,7 +662,9 @@ QSize QLabelPrivate::sizeForWidth(int w) const
             } else {
                 control->setTextWidth(-1);
             }
-            br = QRect(QPoint(0, 0), control->size().toSize());
+
+            QSizeF controlSize = control->size();
+            br = QRect(QPoint(0, 0), QSize(qCeil(controlSize.width()), qCeil(controlSize.height())));
 
             // restore state
             control->setTextWidth(oldTextWidth);
@@ -781,6 +784,95 @@ Qt::TextInteractionFlags QLabel::textInteractionFlags() const
     return d->textInteractionFlags;
 }
 
+/*!
+    Selects text from position \a start and for \a length characters.
+
+    \sa selectedText()
+
+    \bold{Note:} The textInteractionFlags set on the label need to include
+    either TextSelectableByMouse or TextSelectableByKeyboard.
+
+    \since 4.7
+*/
+void QLabel::setSelection(int start, int length)
+{
+    Q_D(QLabel);
+    if (d->control) {
+        d->ensureTextPopulated();
+        QTextCursor cursor = d->control->textCursor();
+        cursor.setPosition(start);
+        cursor.setPosition(start + length, QTextCursor::KeepAnchor);
+        d->control->setTextCursor(cursor);
+    }
+}
+
+/*!
+    \property QLabel::hasSelectedText
+    \brief whether there is any text selected
+
+    hasSelectedText() returns true if some or all of the text has been
+    selected by the user; otherwise returns false.
+
+    By default, this property is false.
+
+    \sa selectedText()
+
+    \bold{Note:} The textInteractionFlags set on the label need to include
+    either TextSelectableByMouse or TextSelectableByKeyboard.
+
+    \since 4.7
+*/
+bool QLabel::hasSelectedText() const
+{
+    Q_D(const QLabel);
+    if (d->control)
+        return d->control->textCursor().hasSelection();
+    return false;
+}
+
+/*!
+    \property QLabel::selectedText
+    \brief the selected text
+
+    If there is no selected text this property's value is
+    an empty string.
+
+    By default, this property contains an empty string.
+
+    \sa hasSelectedText()
+
+    \bold{Note:} The textInteractionFlags set on the label need to include
+    either TextSelectableByMouse or TextSelectableByKeyboard.
+
+    \since 4.7
+*/
+QString QLabel::selectedText() const
+{
+    Q_D(const QLabel);
+    if (d->control)
+        return d->control->textCursor().selectedText();
+    return QString();
+}
+
+/*!
+    selectionStart() returns the index of the first selected character in the
+    label or -1 if no text is selected.
+
+    \sa selectedText()
+
+    \bold{Note:} The textInteractionFlags set on the label need to include
+    either TextSelectableByMouse or TextSelectableByKeyboard.
+
+    \since 4.7
+*/
+int QLabel::selectionStart() const
+{
+    Q_D(const QLabel);
+    if (d->control && d->control->textCursor().hasSelection())
+        return d->control->textCursor().selectionStart();
+    return -1;
+}
+
 /*!\reimp
 */
 QSize QLabel::sizeHint() const
@@ -862,8 +954,8 @@ void QLabel::contextMenuEvent(QContextMenuEvent *ev)
         return;
     }
     ev->accept();
-    menu->exec(ev->globalPos());
-    delete menu;
+    menu->setAttribute(Qt::WA_DeleteOnClose);
+    menu->popup(ev->globalPos());
 #endif
 }
 
@@ -967,7 +1059,8 @@ void QLabel::paintEvent(QPaintEvent *)
     drawFrame(&painter);
     QRect cr = contentsRect();
     cr.adjust(d->margin, d->margin, -d->margin, -d->margin);
-    int align = QStyle::visualAlignment(layoutDirection(), QFlag(d->align));
+    int align = QStyle::visualAlignment(d->isTextLabel ? d->textDirection()
+                                                       : layoutDirection(), QFlag(d->align));
 
 #ifndef QT_NO_MOVIE
     if (d->movie) {
@@ -1027,7 +1120,8 @@ void QLabel::paintEvent(QPaintEvent *)
             d->control->drawContents(&painter, QRectF(), this);
             painter.restore();
         } else {
-            int flags = align;
+            int flags = align | (d->textDirection() == Qt::LeftToRight ? Qt::TextForceLeftToRight
+                                                                       : Qt::TextForceRightToLeft);
             if (d->hasShortcut) {
                 flags |= Qt::TextShowMnemonic;
                 if (!style->styleHint(QStyle::SH_UnderlineShortcut, &opt, this))
@@ -1170,22 +1264,10 @@ void QLabelPrivate::updateShortcut()
     // But then we do want to hide the ampersands, so we can't use shortcutId.
     hasShortcut = false;
 
-    if (control) {
-        ensureTextPopulated();
-        // Underline the first character that follows an ampersand
-        shortcutCursor = control->document()->find(QLatin1String("&"));
-        if (shortcutCursor.isNull())
-            return;
-        hasShortcut = true;
-        shortcutId = q->grabShortcut(QKeySequence::mnemonic(text));
-        shortcutCursor.deleteChar(); // remove the ampersand
-        shortcutCursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
-    } else {
-        if (!text.contains(QLatin1Char('&')))
-            return;
-        hasShortcut = true;
-        shortcutId = q->grabShortcut(QKeySequence::mnemonic(text));
-    }
+    if (!text.contains(QLatin1Char('&')))
+        return;
+    hasShortcut = true;
+    shortcutId = q->grabShortcut(QKeySequence::mnemonic(text));
 }
 
 #endif // QT_NO_SHORTCUT
@@ -1367,10 +1449,6 @@ void QLabel::changeEvent(QEvent *ev)
         d->control->setPalette(palette());
     } else if (ev->type() == QEvent::ContentsRectChange) {
         d->updateLabel();
-    } else if (ev->type() == QEvent::LayoutDirectionChange) {
-        if (d->isTextLabel && d->control) {
-            d->sendControlEvent(ev);
-        }
     }
     QFrame::changeEvent(ev);
 }
@@ -1406,6 +1484,15 @@ void QLabel::setScaledContents(bool enable)
     update(contentsRect());
 }
 
+Qt::LayoutDirection QLabelPrivate::textDirection() const
+{
+    if (control) {
+        QTextOption opt = control->document()->defaultTextOption();
+        return opt.textDirection();
+    }
+
+    return text.isRightToLeft() ? Qt::RightToLeft : Qt::LeftToRight;
+}
 
 /*!
     \fn void QLabel::setAlignment(Qt::AlignmentFlag flag)
@@ -1423,7 +1510,8 @@ QRect QLabelPrivate::documentRect() const
     Q_ASSERT_X(isTextLabel, "documentRect", "document rect called for label that is not a text label!");
     QRect cr = q->contentsRect();
     cr.adjust(margin, margin, -margin, -margin);
-    const int align = QStyle::visualAlignment(q->layoutDirection(), QFlag(this->align));
+    const int align = QStyle::visualAlignment(isTextLabel ? textDirection()
+                                                          : q->layoutDirection(), QFlag(this->align));
     int m = indent;
     if (m < 0 && q->frameWidth()) // no indent, but we do have a frame
         m = q->fontMetrics().width(QLatin1Char('x')) / 2 - margin;
@@ -1456,6 +1544,24 @@ void QLabelPrivate::ensureTextPopulated() const
             doc->setPlainText(text);
 #endif
             doc->setUndoRedoEnabled(false);
+
+#ifndef QT_NO_SHORTCUT
+            if (hasShortcut) {
+                // Underline the first character that follows an ampersand (and remove the others ampersands)
+                int from = 0;
+                bool found = false;
+                QTextCursor cursor;
+                while (!(cursor = control->document()->find((QLatin1String("&")), from)).isNull()) {
+                    cursor.deleteChar(); // remove the ampersand
+                    cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
+                    from = cursor.position();
+                    if (!found && cursor.selectedText() != QLatin1String("&")) { //not a second &
+                        found = true;
+                        shortcutCursor = cursor;
+                    }
+                }
+            }
+#endif
         }
     }
     textDirty = false;
@@ -1466,7 +1572,6 @@ void QLabelPrivate::ensureTextLayouted() const
     if (!textLayoutDirty)
         return;
     ensureTextPopulated();
-    Q_Q(const QLabel);
     if (control) {
         QTextDocument *doc = control->document();
         QTextOption opt = doc->defaultTextOption();
@@ -1478,8 +1583,6 @@ void QLabelPrivate::ensureTextLayouted() const
         else
             opt.setWrapMode(QTextOption::ManualWrap);
 
-        opt.setTextDirection(q->layoutDirection());
-
         doc->setDefaultTextOption(opt);
 
         QTextFrameFormat fmt = doc->rootFrame()->frameFormat();
