@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -39,6 +39,8 @@
 **
 ****************************************************************************/
 
+//#define QT_EXPERIMENTAL_CLIENT_DECORATIONS
+
 #include "qmainwindow.h"
 #include "qmainwindowlayout_p.h"
 
@@ -65,9 +67,6 @@ QT_BEGIN_NAMESPACE
 extern OSWindowRef qt_mac_window_for(const QWidget *); // qwidget_mac.cpp
 QT_END_NAMESPACE
 #endif
-#ifdef QT_SOFTKEYS_ENABLED
-#include <private/qsoftkeymanager_p.h>
-#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -80,9 +79,6 @@ public:
 #ifdef Q_WS_MAC
             , useHIToolBar(false)
 #endif
-#ifdef QT_SOFTKEYS_ENABLED
-            , menuBarAction(0)
-#endif
 #if !defined(QT_NO_DOCKWIDGET) && !defined(QT_NO_CURSOR)
             , hasOldCursor(false) , cursorAdjusted(false)
 #endif
@@ -93,9 +89,6 @@ public:
     Qt::ToolButtonStyle toolButtonStyle;
 #ifdef Q_WS_MAC
     bool useHIToolBar;
-#endif
-#ifdef QT_SOFTKEYS_ENABLED
-    QAction *menuBarAction;
 #endif
     void init();
     QList<int> hoverSeparator;
@@ -108,19 +101,73 @@ public:
     uint hasOldCursor : 1;
     uint cursorAdjusted : 1;
 #endif
+
+    static inline QMainWindowLayout *mainWindowLayout(const QMainWindow *mainWindow)
+    {
+        return mainWindow ? mainWindow->d_func()->layout : static_cast<QMainWindowLayout *>(0);
+    }
 };
 
+QMainWindowLayout *qt_mainwindow_layout(const QMainWindow *mainWindow)
+{
+    return QMainWindowPrivate::mainWindowLayout(mainWindow);
+}
+
+#ifdef QT_EXPERIMENTAL_CLIENT_DECORATIONS
+Q_GUI_EXPORT void qt_setMainWindowTitleWidget(QMainWindow *mainWindow, Qt::DockWidgetArea area, QWidget *widget)
+{
+    QGridLayout *topLayout = qobject_cast<QGridLayout *>(mainWindow->layout());
+    Q_ASSERT(topLayout);
+
+    int row = 0;
+    int column = 0;
+
+    switch (area) {
+    case Qt::LeftDockWidgetArea:
+        row = 1;
+        column = 0;
+        break;
+    case Qt::TopDockWidgetArea:
+        row = 0;
+        column = 1;
+        break;
+    case Qt::BottomDockWidgetArea:
+        row = 2;
+        column = 1;
+        break;
+    case Qt::RightDockWidgetArea:
+        row = 1;
+        column = 2;
+        break;
+    default:
+        Q_ASSERT_X(false, "qt_setMainWindowTitleWidget", "Unknown area");
+        return;
+    }
+
+    if (QLayoutItem *oldItem = topLayout->itemAtPosition(row, column))
+        delete oldItem->widget();
+    topLayout->addWidget(widget, row, column);
+}
+#endif
+
 void QMainWindowPrivate::init()
 {
     Q_Q(QMainWindow);
-    layout = new QMainWindowLayout(q);
+
+#ifdef QT_EXPERIMENTAL_CLIENT_DECORATIONS
+    QGridLayout *topLayout = new QGridLayout(q);
+    topLayout->setContentsMargins(0, 0, 0, 0);
+
+    layout = new QMainWindowLayout(q, topLayout);
+
+    topLayout->addItem(layout, 1, 1);
+#else
+    layout = new QMainWindowLayout(q, 0);
+#endif
+
     const int metric = q->style()->pixelMetric(QStyle::PM_ToolBarIconSize, 0, q);
     iconSize = QSize(metric, metric);
     q->setAttribute(Qt::WA_Hover);
-#ifdef QT_SOFTKEYS_ENABLED
-    menuBarAction = QSoftKeyManager::createAction(QSoftKeyManager::MenuSoftKey, q);
-    menuBarAction->setObjectName(QLatin1String("_q_menuSoftKeyAction"));
-#endif
 }
 
 /*
@@ -474,10 +521,11 @@ QMenuBar *QMainWindow::menuBar() const
 */
 void QMainWindow::setMenuBar(QMenuBar *menuBar)
 {
-    Q_D(QMainWindow);
-    if (d->layout->menuBar() && d->layout->menuBar() != menuBar) {
+    QLayout *topLayout = layout();
+
+    if (topLayout->menuBar() && topLayout->menuBar() != menuBar) {
         // Reparent corner widgets before we delete the old menu bar.
-        QMenuBar *oldMenuBar = qobject_cast<QMenuBar *>(d->layout->menuBar());
+        QMenuBar *oldMenuBar = qobject_cast<QMenuBar *>(topLayout->menuBar());
         if (menuBar) {
             // TopLeftCorner widget.
             QWidget *cornerWidget = oldMenuBar->cornerWidget(Qt::TopLeftCorner);
@@ -491,14 +539,7 @@ void QMainWindow::setMenuBar(QMenuBar *menuBar)
         oldMenuBar->hide();
         oldMenuBar->deleteLater();
     }
-    d->layout->setMenuBar(menuBar);
-
-#ifdef QT_SOFTKEYS_ENABLED
-    if (menuBar)
-        addAction(d->menuBarAction);
-    else
-        removeAction(d->menuBarAction);
-#endif
+    topLayout->setMenuBar(menuBar);
 }
 
 /*!
@@ -1394,6 +1435,9 @@ bool QMainWindow::event(QEvent *event)
 #endif // QT_NO_STATUSTIP
 
         case QEvent::StyleChange:
+#ifndef QT_NO_DOCKWIDGET
+            d->layout->layoutState.dockAreaLayout.styleChangedEvent();
+#endif
             if (!d->explicitIconSize)
                 setIconSize(QSize());
             break;
@@ -1473,7 +1517,8 @@ void QMainWindow::setUnifiedTitleAndToolBarOnMac(bool set)
         return;
 
     // ### Disable the unified toolbar when using anything but the native graphics system.
-    if (windowSurface())
+    // ### Disable when using alien widgets as well
+    if (windowSurface() || testAttribute(Qt::WA_NativeWindow) == false)
         return;
 
     d->useHIToolBar = set;
@@ -1555,11 +1600,15 @@ void QMainWindow::contextMenuEvent(QContextMenuEvent *event)
 
 #ifndef QT_NO_MENU
     QMenu *popup = createPopupMenu();
-    if (popup && !popup->isEmpty()) {
-        popup->exec(event->globalPos());
-        event->accept();
+    if (popup) {
+        if (!popup->isEmpty()) {
+            popup->setAttribute(Qt::WA_DeleteOnClose);
+            popup->popup(event->globalPos());
+            event->accept();
+        } else {
+            delete popup;
+        }
     }
-    delete popup;
 #endif
 }
 #endif // QT_NO_CONTEXTMENU
