@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -160,7 +160,7 @@ static void dumpLayout(QTextStream &qout, const QDockAreaLayout &layout, QString
 
 void qt_dumpLayout(QTextStream &qout, QMainWindow *window)
 {
-    QMainWindowLayout *layout = qobject_cast<QMainWindowLayout*>(window->layout());
+    QMainWindowLayout *layout = qt_mainwindow_layout(window);
     dumpLayout(qout, layout->layoutState.dockAreaLayout, QString());
 }
 
@@ -235,7 +235,7 @@ void QMainWindowLayoutState::apply(bool animated)
     dockAreaLayout.apply(animated);
 #else
     if (centralWidgetItem != 0) {
-        QMainWindowLayout *layout = qobject_cast<QMainWindowLayout*>(mainWindow->layout());
+        QMainWindowLayout *layout = qt_mainwindow_layout(mainWindow);
         Q_ASSERT(layout != 0);
         layout->widgetAnimator.animate(centralWidgetItem->widget(), centralWidgetRect, animated);
     }
@@ -943,7 +943,7 @@ void QMainWindowLayout::toggleToolBarsVisible()
 #ifdef Q_WS_MAC
     if (layoutState.mainWindow->unifiedTitleAndToolBarOnMac()) {
         // If we hit this case, someone has pressed the "toolbar button" which will
-        // toggle the unified toolbar visiblity, because that's what the user wants.
+        // toggle the unified toolbar visibility, because that's what the user wants.
         // We might be in a situation where someone has hidden all the toolbars
         // beforehand (maybe in construction), but now they've hit this button and
         // and are expecting the items to show. What do we do?
@@ -1627,6 +1627,13 @@ void QMainWindowLayout::animationFinished(QWidget *widget)
             tb->d_func()->plug(currentGapRect);
 #endif
 
+        savedState.clear();
+        currentGapPos.clear();
+        pluggingWidget = 0;
+        //applying the state will make sure that the currentGap is updated correctly
+        //and all the geometries (especially the one from the central widget) is correct
+        layoutState.apply(false);
+
 #ifndef QT_NO_DOCKWIDGET
 #ifndef QT_NO_TABBAR
         if (qobject_cast<QDockWidget*>(widget) != 0) {
@@ -1637,13 +1644,6 @@ void QMainWindowLayout::animationFinished(QWidget *widget)
         }
 #endif
 #endif
-
-        savedState.clear();
-        currentGapPos.clear();
-        pluggingWidget = 0;
-        //applying the state will make sure that the currentGap is updated correctly
-        //and all the geometries (especially the one from the central widget) is correct
-        layoutState.apply(false);
     }
 
     if (!widgetAnimator.animating()) {
@@ -1674,8 +1674,8 @@ void QMainWindowLayout::restore(bool keepSavedState)
     updateGapIndicator();
 }
 
-QMainWindowLayout::QMainWindowLayout(QMainWindow *mainwindow)
-    : QLayout(mainwindow)
+QMainWindowLayout::QMainWindowLayout(QMainWindow *mainwindow, QLayout *parentLayout)
+    : QLayout(parentLayout ? static_cast<QWidget *>(0) : mainwindow)
     , layoutState(mainwindow)
     , savedState(mainwindow)
     , dockOptions(QMainWindow::AnimatedDocks | QMainWindow::AllowTabbedDocks)
@@ -1698,6 +1698,9 @@ QMainWindowLayout::QMainWindowLayout(QMainWindow *mainwindow)
     , blockVisiblityCheck(false)
 #endif
 {
+    if (parentLayout)
+        setParent(parentLayout);
+
 #ifndef QT_NO_DOCKWIDGET
 #ifndef QT_NO_TABBAR
     sep = mainwindow->style()->pixelMetric(QStyle::PM_DockWidgetSeparatorExtent, 0, mainwindow);
@@ -1772,6 +1775,7 @@ void QMainWindowLayout::setCentralWidget(QWidget *widget)
     if (savedState.isValid()) {
 #ifndef QT_NO_DOCKWIDGET
         savedState.dockAreaLayout.centralWidgetItem = layoutState.dockAreaLayout.centralWidgetItem;
+        savedState.dockAreaLayout.fallbackToSizeHints = true;
 #else
         savedState.centralWidgetItem = layoutState.centralWidgetItem;
 #endif
