@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -82,6 +82,9 @@ static void qt_mac_updateToolBarButtonHint(QWidget *parentWidget)
 }
 #endif
 
+// qmainwindow.cpp
+extern QMainWindowLayout *qt_mainwindow_layout(const QMainWindow *window);
+
 /******************************************************************************
 ** QToolBarPrivate
 */
@@ -197,7 +200,7 @@ void QToolBarPrivate::initDrag(const QPoint &pos)
 
     QMainWindow *win = qobject_cast<QMainWindow*>(parent);
     Q_ASSERT(win != 0);
-    QMainWindowLayout *layout = qobject_cast<QMainWindowLayout*>(win->layout());
+    QMainWindowLayout *layout = qt_mainwindow_layout(win);
     Q_ASSERT(layout != 0);
     if (layout->pluggingWidget != 0) // the main window is animating a docking operation
         return;
@@ -223,7 +226,7 @@ void QToolBarPrivate::startDrag(bool moving)
 
     QMainWindow *win = qobject_cast<QMainWindow*>(parent);
     Q_ASSERT(win != 0);
-    QMainWindowLayout *layout = qobject_cast<QMainWindowLayout*>(win->layout());
+    QMainWindowLayout *layout = qt_mainwindow_layout(win);
     Q_ASSERT(layout != 0);
 
     if (!moving) {
@@ -247,8 +250,7 @@ void QToolBarPrivate::endDrag()
     q->releaseMouse();
 
     if (state->dragging) {
-        QMainWindowLayout *layout =
-            qobject_cast<QMainWindowLayout *>(q->parentWidget()->layout());
+        QMainWindowLayout *layout = qt_mainwindow_layout(qobject_cast<QMainWindow *>(q->parentWidget()));
         Q_ASSERT(layout != 0);
 
         if (!layout->plug(state->widgetItem)) {
@@ -340,7 +342,7 @@ bool QToolBarPrivate::mouseMoveEvent(QMouseEvent *event)
     if (win == 0)
         return true;
 
-    QMainWindowLayout *layout = qobject_cast<QMainWindowLayout*>(win->layout());
+    QMainWindowLayout *layout = qt_mainwindow_layout(win);
     Q_ASSERT(layout != 0);
 
     if (layout->pluggingWidget == 0
@@ -396,10 +398,10 @@ bool QToolBarPrivate::mouseMoveEvent(QMouseEvent *event)
 void QToolBarPrivate::unplug(const QRect &_r)
 {
     Q_Q(QToolBar);
-    layout->setExpanded(false, false);
     QRect r = _r;
     r.moveTopLeft(q->mapToGlobal(QPoint(0, 0)));
     setWindowState(true, true, r);
+    layout->setExpanded(false);
 }
 
 void QToolBarPrivate::plug(const QRect &r)
@@ -441,8 +443,7 @@ void QToolBarPrivate::plug(const QRect &r)
     When a QToolBar is not a child of a QMainWindow, it looses the ability
     to populate the extension pop up with widgets added to the toolbar using
     addWidget(). Please use widget actions created by inheriting QWidgetAction
-    and implementing QWidgetAction::createWidget() instead. This is a known
-    issue which will be fixed in a future release.
+    and implementing QWidgetAction::createWidget() instead.
 
     \sa QToolButton, QMenu, QAction, {Application Example}
 */
@@ -533,6 +534,14 @@ void QToolBarPrivate::plug(const QRect &r)
 */
 
 
+/*!
+    \fn void QToolBar::visibilityChanged(bool visible)
+    \since 4.7
+
+    This signal is emitted when the toolbar becomes \a visible (or
+    invisible). This happens when the widget is hidden or shown.
+*/
+
 /*!
     Constructs a QToolBar with the given \a parent.
 */
@@ -581,7 +590,7 @@ QToolBar::~QToolBar()
     QMainWindow *mainwindow = qobject_cast<QMainWindow *>(parentWidget());
     if (mainwindow) {
 #ifdef Q_WS_MAC
-        QMainWindowLayout *mainwin_layout = qobject_cast<QMainWindowLayout *>(mainwindow->layout());
+        QMainWindowLayout *mainwin_layout = qt_mainwindow_layout(mainwindow);
         if (mainwin_layout && mainwin_layout->layoutState.toolBarAreaLayout.isEmpty()
                 && mainwindow->testAttribute(Qt::WA_WState_Created))
             macWindowToolbarShow(mainwindow, false);
@@ -1123,11 +1132,12 @@ bool QToolBar::event(QEvent *event)
         // fallthrough intended
     case QEvent::Show:
         d->toggleViewAction->setChecked(event->type() == QEvent::Show);
+        emit visibilityChanged(event->type() == QEvent::Show);
 #if defined(Q_WS_MAC)
         if (toolbarInUnifiedToolBar(this)) {
              // I can static_cast because I did the qobject_cast in the if above, therefore
             // we must have a QMainWindowLayout here.
-            QMainWindowLayout *mwLayout = static_cast<QMainWindowLayout *>(parentWidget()->layout());
+            QMainWindowLayout *mwLayout = qt_mainwindow_layout(qobject_cast<QMainWindow *>(parentWidget()));
             mwLayout->fixSizeInUnifiedToolbar(this);
             mwLayout->syncUnifiedToolbarVisibility();
         }
@@ -1281,6 +1291,8 @@ QWidget *QToolBar::widgetForAction(QAction *action) const
     return d->layout->itemAt(index)->widget();
 }
 
+extern QMainWindowLayout *qt_mainwindow_layout(const QMainWindow *window);
+
 /*!
     \internal
 */
@@ -1307,7 +1319,7 @@ void QToolBar::initStyleOption(QStyleOptionToolBar *option) const
     if (!mainWindow)
         return;
 
-    QMainWindowLayout *layout = qobject_cast<QMainWindowLayout *>(mainWindow->layout());
+    QMainWindowLayout *layout = qt_mainwindow_layout(mainWindow);
     Q_ASSERT_X(layout != 0, "QToolBar::initStyleOption()",
                "QMainWindow->layout() != QMainWindowLayout");
 
