@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -57,6 +57,9 @@
 
 QT_BEGIN_NAMESPACE
 
+// qmainwindow.cpp
+extern QMainWindowLayout *qt_mainwindow_layout(const QMainWindow *window);
+
 /******************************************************************************
 ** QToolBarItem
 */
@@ -404,7 +407,7 @@ void QToolBarLayout::setGeometry(const QRect &rect)
     if (QMainWindow *win = qobject_cast<QMainWindow*>(tb->parentWidget())) {
         Qt::ToolBarArea area = win->toolBarArea(tb);
         if (win->unifiedTitleAndToolBarOnMac() && area == Qt::TopToolBarArea) {
-            static_cast<QMainWindowLayout *>(win->layout())->fixSizeInUnifiedToolbar(tb);
+            qt_mainwindow_layout(win)->fixSizeInUnifiedToolbar(tb);
         }
     }
 #  endif
@@ -642,7 +645,7 @@ QSize QToolBarLayout::expandedSize(const QSize &size) const
     return result;
 }
 
-void QToolBarLayout::setExpanded(bool exp, bool animated)
+void QToolBarLayout::setExpanded(bool exp)
 {
     if (exp == expanded)
         return;
@@ -654,8 +657,12 @@ void QToolBarLayout::setExpanded(bool exp, bool animated)
     if (!tb)
         return;
     if (QMainWindow *win = qobject_cast<QMainWindow*>(tb->parentWidget())) {
-        animating = true;
-        QMainWindowLayout *layout = qobject_cast<QMainWindowLayout*>(win->layout());
+#ifdef QT_NO_DOCKWIDGET
+        animating = false;
+#else
+        animating = !tb->isWindow() && win->isAnimated();
+#endif
+        QMainWindowLayout *layout = qt_mainwindow_layout(win);
         if (expanded) {
             tb->raise();
         } else {
@@ -665,7 +672,7 @@ void QToolBarLayout::setExpanded(bool exp, bool animated)
                 layoutActions(rect.size());
             }
         }
-        layout->layoutState.toolBarAreaLayout.apply(animated);
+        layout->layoutState.toolBarAreaLayout.apply(animating);
     }
 }
 
