@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -125,6 +125,11 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     encrypted channel could not be established. The sslErrors() signal
     should have been emitted.
 
+    \value TemporaryNetworkFailureError the connection was broken due
+    to disconnection from the network, however the system has initiated
+    roaming to another access point. The request should be resubmitted
+    and will be processed as soon as the connection is re-established.
+
     \value ProxyConnectionRefusedError the connection to the proxy
     server was refused (the proxy server is not accepting requests)
 
@@ -171,7 +176,7 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     \value UnknownProxyError            an unknown proxy-related error
     was detected
 
-    \value UnknownContentError          an unknonwn error related to
+    \value UnknownContentError          an unknown error related to
     the remote content was detected
 
     \value ProtocolFailure              a breakdown in protocol was
@@ -239,7 +244,10 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     \note Do not delete the object in the slot connected to this
     signal. Use deleteLater().
 
-    \sa QNetworkAccessManager::finished()
+    You can also use isFinished() to check if a QNetworkReply
+    has finished even before you receive the finished() signal.
+
+    \sa QNetworkAccessManager::finished(), isFinished()
 */
 
 /*!
@@ -275,9 +283,6 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     The upload is finished when \a bytesSent is equal to \a
     bytesTotal. At that time, \a bytesTotal will not be -1.
 
-    This signal is suitable to connecting to QProgressBar::setValue()
-    to update the QProgressBar that provides user feedback.
-
     \sa downloadProgress()
 */
 
@@ -298,9 +303,6 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     The download is finished when \a bytesReceived is equal to \a
     bytesTotal. At that time, \a bytesTotal will not be -1.
 
-    This signal is suitable to connecting to QProgressBar::setValue()
-    to update the QProgressBar that provides user feedback.
-
     Note that the values of both \a bytesReceived and \a bytesTotal
     may be different from size(), the total number of bytes
     obtained through read() or readAll(), or the value of the
@@ -530,6 +532,21 @@ QByteArray QNetworkReply::rawHeader(const QByteArray &headerName) const
     return QByteArray();
 }
 
+/*! \typedef QNetworkReply::RawHeaderPair
+
+  RawHeaderPair is a QPair<QByteArray, QByteArray> where the first
+  QByteArray is the header name and the second is the header.
+ */
+
+/*!
+  Returns a list of raw header pairs.
+ */
+const QList<QNetworkReply::RawHeaderPair>& QNetworkReply::rawHeaderPairs() const
+{
+    Q_D(const QNetworkReply);
+    return d->rawHeaders;
+}
+
 /*!
     Returns a list of headers fields that were sent by the remote
     server, in the order that they were sent. Duplicate headers are
