@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -116,6 +116,68 @@ extern void qt_mac_dispose_rgn(RgnHandle); //qregion_mac.cpp
 extern QRegion qt_mac_convert_mac_region(RgnHandle); //qregion_mac.cpp
 extern void qt_mac_to_pascal_string(QString s, Str255 str, TextEncoding encoding=0, int len=-1);  //qglobal.cpp
 
+/*
+    QGLTemporaryContext implementation
+*/
+
+class QGLTemporaryContextPrivate
+{
+public:
+#ifndef QT_MAC_USE_COCOA
+    AGLContext ctx;
+#else
+    NSOpenGLContext *ctx;
+#endif
+};
+
+QGLTemporaryContext::QGLTemporaryContext(bool, QWidget *)
+    : d(new QGLTemporaryContextPrivate)
+{
+    d->ctx = 0;
+#ifndef QT_MAC_USE_COCOA
+    GLint attribs[] = {AGL_RGBA, AGL_NONE};
+    AGLPixelFormat fmt = aglChoosePixelFormat(0, 0, attribs);
+    if (!fmt) {
+        qDebug("QGLTemporaryContext: Couldn't find any RGB visuals");
+        return;
+    }
+    d->ctx = aglCreateContext(fmt, 0);
+    if (!d->ctx)
+        qDebug("QGLTemporaryContext: Unable to create context");
+    else
+        aglSetCurrentContext(d->ctx);
+    aglDestroyPixelFormat(fmt);
+#else
+    QMacCocoaAutoReleasePool pool;
+    NSOpenGLPixelFormatAttribute attribs[] = { 0 };
+    NSOpenGLPixelFormat *fmt = [[NSOpenGLPixelFormat alloc] initWithAttributes:attribs];
+    if (!fmt) {
+        qWarning("QGLTemporaryContext: Cannot find any visuals");
+        return;
+    }
+
+    d->ctx = [[NSOpenGLContext alloc] initWithFormat:fmt shareContext:0];
+    if (!d->ctx)
+        qWarning("QGLTemporaryContext: Cannot create context");
+    else
+        [d->ctx makeCurrentContext];
+    [fmt release];
+#endif
+}
+
+QGLTemporaryContext::~QGLTemporaryContext()
+{
+    if (d->ctx) {
+#ifndef QT_MAC_USE_COCOA
+        aglSetCurrentContext(0);
+        aglDestroyContext(d->ctx);
+#else
+        [NSOpenGLContext clearCurrentContext];
+        [d->ctx release];
+#endif
+    }
+}
+
 bool QGLFormat::hasOpenGL()
 {
     return true;
@@ -460,6 +522,7 @@ void QGLContext::reset()
     if (d->cx)
         aglDestroyContext((AGLContext)d->cx);
 #else
+    QMacCocoaAutoReleasePool pool;
     [static_cast<NSOpenGLContext *>(d->cx) release];
 #endif
     d->cx = 0;
@@ -475,7 +538,7 @@ void QGLContext::reset()
     d->valid = false;
     d->transpColor = QColor();
     d->initDone = false;
-    qgl_share_reg()->removeShare(this);
+    QGLContextGroup::removeShare(this);
 }
 
 void QGLContext::makeCurrent()
@@ -605,10 +668,22 @@ void QGLContext::updatePaintDevice()
             }
         }
     } else if (d->paintDevice->devType() == QInternal::Pixmap) {
-        QPixmap *pm = (QPixmap *)d->paintDevice;
-        PixMapHandle mac_pm = GetGWorldPixMap((GWorldPtr)pm->macQDHandle());
-        aglSetOffScreen((AGLContext)d->cx, pm->width(), pm->height(),
-                GetPixRowBytes(mac_pm), GetPixBaseAddr(mac_pm));
+        QPixmap *pm = reinterpret_cast<QPixmap *>(d->paintDevice);
+
+        unsigned long qdformat = k32ARGBPixelFormat;
+        if (QSysInfo::ByteOrder == QSysInfo::LittleEndian)
+            qdformat = k32BGRAPixelFormat;
+        Rect rect;
+        SetRect(&rect, 0, 0, pm->width(), pm->height());
+
+        GWorldPtr gworld;
+        NewGWorldFromPtr(&gworld, qdformat, &rect, 0, 0, 0,
+                         reinterpret_cast<char *>(qt_mac_pixmap_get_base(pm)), 
+                         qt_mac_pixmap_get_bytes_per_line(pm));
+
+        PixMapHandle pixmapHandle = GetGWorldPixMap(gworld);
+        aglSetOffScreen(reinterpret_cast<AGLContext>(d->cx), pm->width(), pm->height(),
+                        GetPixRowBytes(pixmapHandle), GetPixBaseAddr(pixmapHandle));
     } else {
         qWarning("QGLContext::updatePaintDevice(): Not sure how to render OpenGL on this device!");
     }
@@ -622,9 +697,17 @@ void QGLContext::updatePaintDevice()
         QWidget *w = (QWidget *)d->paintDevice;
         NSView *view = qt_mac_nativeview_for(w);
 
-        // ideally we would use QWidget::isVisible(), but we get "invalid drawable" errors
-        if (![(NSWindow *)qt_mac_window_for(w) isVisible])
-            return;
+        // Trying to attach the GL context to the NSView will fail with
+        // "invalid drawable" if done too soon, but we have to make sure
+        // the connection is made before the first paint event. Using
+        // the NSView do to this check fails as the NSView is visible
+        // before it's safe to connect, and using the NSWindow fails as
+        // the NSWindow will become visible after the first paint event.
+        // This leaves us with the QWidget, who's visible state seems
+        // to match the point in time when it's safe to connect.
+        if (!w || !w->isVisible())
+            return; // Not safe to attach GL context to view yet
+
         if ([static_cast<NSOpenGLContext *>(d->cx) view] != view && ![view isHidden])
             [static_cast<NSOpenGLContext *>(d->cx) setView:view];
     } else if (d->paintDevice->devType() == QInternal::Pixmap) {
@@ -729,17 +812,22 @@ void QGLContext::generateFontDisplayLists(const QFont & /* fnt */, int /* listBa
 static CFBundleRef qt_getOpenGLBundle()
 {
     CFBundleRef bundle = 0;
+    CFStringRef urlString = QCFString::toCFStringRef(QLatin1String("/System/Library/Frameworks/OpenGL.framework"));
     QCFType<CFURLRef> url = CFURLCreateWithFileSystemPath(kCFAllocatorDefault,
-                 QCFString::toCFStringRef(QLatin1String("/System/Library/Frameworks/OpenGL.framework")), kCFURLPOSIXPathStyle, false);
+                 urlString, kCFURLPOSIXPathStyle, false);
     if (url)
         bundle = CFBundleCreate(kCFAllocatorDefault, url);
+    CFRelease(urlString);
     return bundle;
 }
 
 void *QGLContext::getProcAddress(const QString &proc) const
 {
-    return CFBundleGetFunctionPointerForName(QCFType<CFBundleRef>(qt_getOpenGLBundle()),
-                                             QCFString(proc));
+    CFStringRef procName = QCFString(proc).toCFStringRef(proc);
+    void *result = CFBundleGetFunctionPointerForName(QCFType<CFBundleRef>(qt_getOpenGLBundle()),
+                                             procName);
+    CFRelease(procName);
+    return result;
 }
 #ifndef QT_MAC_USE_COCOA
 /*****************************************************************************
@@ -876,8 +964,6 @@ void QGLWidgetPrivate::init(QGLContext *context, const QGLWidget *shareWidget)
             break;
         current = current->parentWidget();
     }
-
-    isGLWidget = 1;
 }
 
 bool QGLWidgetPrivate::renderCxPm(QPixmap*)
@@ -905,54 +991,6 @@ void QGLWidgetPrivate::updatePaintDevice()
     q->update();
 }
 
-
-void QGLExtensions::init()
-{
-    static bool init_done = false;
-
-    if (init_done)
-        return;
-    init_done = true;
-
-#ifndef QT_MAC_USE_COCOA
-    GLint attribs[] = { AGL_RGBA, AGL_NONE };
-    AGLPixelFormat fmt = aglChoosePixelFormat(0, 0, attribs);
-    if (!fmt) {
-        qDebug("QGLExtensions: Couldn't find any RGB visuals");
-        return;
-    }
-    AGLContext ctx = aglCreateContext(fmt, 0);
-    if (!ctx) {
-        qDebug("QGLExtensions: Unable to create context");
-    } else {
-        aglSetCurrentContext(ctx);
-        init_extensions();
-        aglSetCurrentContext(0);
-        aglDestroyContext(ctx);
-    }
-    aglDestroyPixelFormat(fmt);
-#else
-    QMacCocoaAutoReleasePool pool;
-    NSOpenGLPixelFormatAttribute attribs[] = { 0 };
-    NSOpenGLPixelFormat *fmt = [[NSOpenGLPixelFormat alloc] initWithAttributes:attribs];
-    if (!fmt) {
-        qWarning("QGLExtensions: Cannot find any visuals");
-        return;
-    }
-
-    NSOpenGLContext *ctx = [[NSOpenGLContext alloc] initWithFormat:fmt shareContext:0];
-    if (!ctx) {
-        qWarning("QGLExtensions: Cannot create context");
-    } else {
-        [ctx makeCurrentContext];
-        init_extensions();
-        [NSOpenGLContext clearCurrentContext];
-        [ctx release];
-    }
-    [fmt release];
-#endif
-}
-
 #endif
 
 QT_END_NAMESPACE
