@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -724,7 +724,8 @@ public:
         if (start.isValid()) {
             m_current = start;
         } else if (m_view && m_view->model()) {
-            m_current = view->model()->index(0, 0);
+            m_current = view->rootIndex().isValid() ? 
+                        view->rootIndex().child(0,0) : view->model()->index(0, 0);
         }
     }
 
@@ -1511,10 +1512,14 @@ QString QAccessibleTabBar::text(Text t, int child) const
         default:
             break;
         }
-    } else if (child > 0) {
+    } else {
         switch (t) {
         case Name:
-            return qt_accStripAmp(tabBar()->tabText(child - 1));
+            if (child > 0)
+                return qt_accStripAmp(tabBar()->tabText(child - 1));
+            else if (tabBar()->currentIndex() != -1)
+                return qt_accStripAmp(tabBar()->tabText(tabBar()->currentIndex()));
+            break;
         default:
             break;
         }
