@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -131,8 +131,8 @@ QDirectFBScreenPrivate::QDirectFBScreenPrivate(QDirectFBScreen *qptr)
     QWSSignalHandler::instance()->addObject(this);
 #endif
 #ifdef QT_DIRECTFB_WM
-    connect(QWSServer::instance(), SIGNAL(windowEvent(QWSWindow*, QWSServer::WindowEvent)),
-            this, SLOT(onWindowEvent(QWSWindow*, QWSServer::WindowEvent)));
+    connect(QWSServer::instance(), SIGNAL(windowEvent(QWSWindow*,QWSServer::WindowEvent)),
+            this, SLOT(onWindowEvent(QWSWindow*,QWSServer::WindowEvent)));
 #endif
 }
 
@@ -921,7 +921,7 @@ void QDirectFBScreenPrivate::setFlipFlags(const QStringList &args)
                          qPrintable(flip));
         }
     } else {
-        flipFlags = DSFLIP_BLIT;
+        flipFlags = DSFLIP_BLIT|DSFLIP_ONSYNC;
     }
 }
 
@@ -958,47 +958,47 @@ struct FlagDescription {
 };
 
 static const FlagDescription accelerationDescriptions[] = {
-    { " DFXL_NONE ", DFXL_NONE },
-    { " DFXL_FILLRECTANGLE", DFXL_FILLRECTANGLE },
-    { " DFXL_DRAWRECTANGLE", DFXL_DRAWRECTANGLE },
-    { " DFXL_DRAWLINE", DFXL_DRAWLINE },
-    { " DFXL_FILLTRIANGLE", DFXL_FILLTRIANGLE },
-    { " DFXL_BLIT", DFXL_BLIT },
-    { " DFXL_STRETCHBLIT", DFXL_STRETCHBLIT },
-    { " DFXL_TEXTRIANGLES", DFXL_TEXTRIANGLES },
-    { " DFXL_DRAWSTRING", DFXL_DRAWSTRING },
+    { "DFXL_NONE", DFXL_NONE },
+    { "DFXL_FILLRECTANGLE", DFXL_FILLRECTANGLE },
+    { "DFXL_DRAWRECTANGLE", DFXL_DRAWRECTANGLE },
+    { "DFXL_DRAWLINE", DFXL_DRAWLINE },
+    { "DFXL_FILLTRIANGLE", DFXL_FILLTRIANGLE },
+    { "DFXL_BLIT", DFXL_BLIT },
+    { "DFXL_STRETCHBLIT", DFXL_STRETCHBLIT },
+    { "DFXL_TEXTRIANGLES", DFXL_TEXTRIANGLES },
+    { "DFXL_DRAWSTRING", DFXL_DRAWSTRING },
     { 0, 0 }
 };
 
 static const FlagDescription blitDescriptions[] = {
-    { " DSBLIT_NOFX", DSBLIT_NOFX },
-    { " DSBLIT_BLEND_ALPHACHANNEL", DSBLIT_BLEND_ALPHACHANNEL },
-    { " DSBLIT_BLEND_COLORALPHA", DSBLIT_BLEND_COLORALPHA },
-    { " DSBLIT_COLORIZE", DSBLIT_COLORIZE },
-    { " DSBLIT_SRC_COLORKEY", DSBLIT_SRC_COLORKEY },
-    { " DSBLIT_DST_COLORKEY", DSBLIT_DST_COLORKEY },
-    { " DSBLIT_SRC_PREMULTIPLY", DSBLIT_SRC_PREMULTIPLY },
-    { " DSBLIT_DST_PREMULTIPLY", DSBLIT_DST_PREMULTIPLY },
-    { " DSBLIT_DEMULTIPLY", DSBLIT_DEMULTIPLY },
-    { " DSBLIT_DEINTERLACE", DSBLIT_DEINTERLACE },
+    { "DSBLIT_NOFX", DSBLIT_NOFX },
+    { "DSBLIT_BLEND_ALPHACHANNEL", DSBLIT_BLEND_ALPHACHANNEL },
+    { "DSBLIT_BLEND_COLORALPHA", DSBLIT_BLEND_COLORALPHA },
+    { "DSBLIT_COLORIZE", DSBLIT_COLORIZE },
+    { "DSBLIT_SRC_COLORKEY", DSBLIT_SRC_COLORKEY },
+    { "DSBLIT_DST_COLORKEY", DSBLIT_DST_COLORKEY },
+    { "DSBLIT_SRC_PREMULTIPLY", DSBLIT_SRC_PREMULTIPLY },
+    { "DSBLIT_DST_PREMULTIPLY", DSBLIT_DST_PREMULTIPLY },
+    { "DSBLIT_DEMULTIPLY", DSBLIT_DEMULTIPLY },
+    { "DSBLIT_DEINTERLACE", DSBLIT_DEINTERLACE },
 #if (Q_DIRECTFB_VERSION >= 0x000923)
-    { " DSBLIT_SRC_PREMULTCOLOR", DSBLIT_SRC_PREMULTCOLOR },
-    { " DSBLIT_XOR", DSBLIT_XOR },
+    { "DSBLIT_SRC_PREMULTCOLOR", DSBLIT_SRC_PREMULTCOLOR },
+    { "DSBLIT_XOR", DSBLIT_XOR },
 #endif
 #if (Q_DIRECTFB_VERSION >= 0x010000)
-    { " DSBLIT_INDEX_TRANSLATION", DSBLIT_INDEX_TRANSLATION },
+    { "DSBLIT_INDEX_TRANSLATION", DSBLIT_INDEX_TRANSLATION },
 #endif
     { 0, 0 }
 };
 
 static const FlagDescription drawDescriptions[] = {
-    { " DSDRAW_NOFX", DSDRAW_NOFX },
-    { " DSDRAW_BLEND", DSDRAW_BLEND },
-    { " DSDRAW_DST_COLORKEY", DSDRAW_DST_COLORKEY },
-    { " DSDRAW_SRC_PREMULTIPLY", DSDRAW_SRC_PREMULTIPLY },
-    { " DSDRAW_DST_PREMULTIPLY", DSDRAW_DST_PREMULTIPLY },
-    { " DSDRAW_DEMULTIPLY", DSDRAW_DEMULTIPLY },
-    { " DSDRAW_XOR", DSDRAW_XOR },
+    { "DSDRAW_NOFX", DSDRAW_NOFX },
+    { "DSDRAW_BLEND", DSDRAW_BLEND },
+    { "DSDRAW_DST_COLORKEY", DSDRAW_DST_COLORKEY },
+    { "DSDRAW_SRC_PREMULTIPLY", DSDRAW_SRC_PREMULTIPLY },
+    { "DSDRAW_DST_PREMULTIPLY", DSDRAW_DST_PREMULTIPLY },
+    { "DSDRAW_DEMULTIPLY", DSDRAW_DEMULTIPLY },
+    { "DSDRAW_XOR", DSDRAW_XOR },
     { 0, 0 }
 };
 #endif
@@ -1063,7 +1063,7 @@ static inline bool setIntOption(const QStringList &arguments, const QString &var
 
 static inline QColor colorFromName(const QString &name)
 {
-    QRegExp rx("#([0-9a-f][0-9a-f])([0-9a-f][0-9a-f])([0-9a-f][0-9a-f])([0-9a-f][0-9a-f])");
+    QRegExp rx(QLatin1String("#([0-9a-f][0-9a-f])([0-9a-f][0-9a-f])([0-9a-f][0-9a-f])([0-9a-f][0-9a-f])"));
     rx.setCaseSensitivity(Qt::CaseInsensitive);
     if (rx.exactMatch(name)) {
         Q_ASSERT(rx.captureCount() == 4);
@@ -1126,6 +1126,8 @@ bool QDirectFBScreen::connect(const QString &displaySpec)
 
     if (displayArgs.contains(QLatin1String("boundingrectflip"), Qt::CaseInsensitive)) {
         d_ptr->directFBFlags |= BoundingRectFlip;
+    } else if (displayArgs.contains(QLatin1String("nopartialflip"), Qt::CaseInsensitive)) {
+        d_ptr->directFBFlags |= NoPartialFlip;
     }
 
 #ifdef QT_DIRECTFB_IMAGECACHE
@@ -1139,6 +1141,8 @@ bool QDirectFBScreen::connect(const QString &displaySpec)
 #endif
         d_ptr->dfb->SetCooperativeLevel(d_ptr->dfb, DFSCL_FULLSCREEN);
 
+    const bool forcePremultiplied = displayArgs.contains(QLatin1String("forcepremultiplied"), Qt::CaseInsensitive);
+
     DFBSurfaceDescription description;
     memset(&description, 0, sizeof(DFBSurfaceDescription));
     IDirectFBSurface *surface;
@@ -1167,7 +1171,7 @@ bool QDirectFBScreen::connect(const QString &displaySpec)
             description.caps |= capabilities[i].cap;
     }
 
-    if (displayArgs.contains(QLatin1String("forcepremultiplied"), Qt::CaseInsensitive)) {
+    if (forcePremultiplied) {
         description.caps |= DSCAPS_PREMULTIPLIED;
     }
 
@@ -1217,6 +1221,8 @@ bool QDirectFBScreen::connect(const QString &displaySpec)
         d_ptr->alphaPixmapFormat = QImage::Format_ARGB32_Premultiplied;
         break;
     case QImage::Format_ARGB32:
+        if (forcePremultiplied)
+            d_ptr->alphaPixmapFormat = pixelFormat = QImage::Format_ARGB32_Premultiplied;
     case QImage::Format_ARGB32_Premultiplied:
     case QImage::Format_ARGB4444_Premultiplied:
     case QImage::Format_ARGB8555_Premultiplied:
@@ -1259,11 +1265,14 @@ bool QDirectFBScreen::connect(const QString &displaySpec)
     setIntOption(displayArgs, QLatin1String("height"), &h);
 
 #ifndef QT_NO_DIRECTFB_LAYER
-    result = d_ptr->dfb->GetDisplayLayer(d_ptr->dfb, DLID_PRIMARY,
+    int layerId = DLID_PRIMARY;
+    setIntOption(displayArgs, QLatin1String("layerid"), &layerId);
+
+    result = d_ptr->dfb->GetDisplayLayer(d_ptr->dfb, static_cast<DFBDisplayLayerID>(layerId),
                                          &d_ptr->dfbLayer);
     if (result != DFB_OK) {
         DirectFBError("QDirectFBScreen::connect: "
-                      "Unable to get primary display layer!", result);
+                      "Unable to get display layer!", result);
         return false;
     }
     result = d_ptr->dfbLayer->GetScreen(d_ptr->dfbLayer, &d_ptr->dfbScreen);
@@ -1275,7 +1284,14 @@ bool QDirectFBScreen::connect(const QString &displaySpec)
 #ifdef QT_NO_DIRECTFB_WM
         result = d_ptr->primarySurface->GetSize(d_ptr->primarySurface, &w, &h);
 #elif (Q_DIRECTFB_VERSION >= 0x010000)
-        result = d_ptr->dfbScreen->GetSize(d_ptr->dfbScreen, &w, &h);
+        IDirectFBSurface *layerSurface;
+        if (d_ptr->dfbLayer->GetSurface(d_ptr->dfbLayer, &layerSurface) != DFB_OK) {
+            result = layerSurface->GetSize(layerSurface, &w, &h);
+            layerSurface->Release(layerSurface);
+        }
+        if (w <= 0 || h <= 0) {
+            result = d_ptr->dfbScreen->GetSize(d_ptr->dfbScreen, &w, &h);
+        }
 #else
         qWarning("QDirectFBScreen::connect: DirectFB versions prior to 1.0 do not offer a way\n"
                  "query the size of the primary surface in windowed mode. You have to specify\n"
@@ -1664,7 +1680,7 @@ bool QDirectFBScreen::initSurfaceDescriptionPixelFormat(DFBSurfaceDescription *d
 
 uchar *QDirectFBScreen::lockSurface(IDirectFBSurface *surface, DFBSurfaceLockFlags flags, int *bpl)
 {
-    void *mem;
+    void *mem = 0;
     const DFBResult result = surface->Lock(surface, flags, &mem, bpl);
     if (result != DFB_OK) {
         DirectFBError("QDirectFBScreen::lockSurface()", result);
@@ -1673,11 +1689,22 @@ uchar *QDirectFBScreen::lockSurface(IDirectFBSurface *surface, DFBSurfaceLockFla
     return reinterpret_cast<uchar*>(mem);
 }
 
+static inline bool isFullUpdate(IDirectFBSurface *surface, const QRegion &region, const QPoint &offset)
+{
+    if (offset == QPoint(0, 0) && region.rectCount() == 1) {
+	QSize size;
+	surface->GetSize(surface, &size.rwidth(), &size.rheight());
+	if (region.boundingRect().size() == size)
+	    return true;
+    }
+    return false;
+}
 
 void QDirectFBScreen::flipSurface(IDirectFBSurface *surface, DFBSurfaceFlipFlags flipFlags,
                                   const QRegion &region, const QPoint &offset)
 {
-    if (!(flipFlags & DSFLIP_BLIT)) {
+    if (d_ptr->directFBFlags & NoPartialFlip
+        || (!(flipFlags & DSFLIP_BLIT) && QT_PREPEND_NAMESPACE(isFullUpdate(surface, region, offset)))) {
         surface->Flip(surface, 0, flipFlags);
     } else {
         if (!(d_ptr->directFBFlags & BoundingRectFlip) && region.rectCount() > 1) {
@@ -1763,7 +1790,6 @@ IDirectFBSurface *QDirectFBScreen::subSurfaceForWidget(const QWidget *widget, co
 }
 #endif
 
-#ifndef QT_DIRECTFB_PLUGIN
 Q_GUI_EXPORT IDirectFBSurface *qt_directfb_surface_for_widget(const QWidget *widget, QRect *rect)
 {
     return QDirectFBScreen::instance() ? QDirectFBScreen::instance()->surfaceForWidget(widget, rect) : 0;
@@ -1780,7 +1806,6 @@ Q_GUI_EXPORT IDirectFBWindow *qt_directfb_window_for_widget(const QWidget *widge
     return QDirectFBScreen::instance() ? QDirectFBScreen::instance()->windowForWidget(widget) : 0;
 }
 
-#endif
 #endif
 
 QT_END_NAMESPACE
