@@ -55,7 +55,7 @@ struct QMetaObject;
 
 struct Type
 {
-    enum ReferenceType { NoReference, Reference, Pointer };
+    enum ReferenceType { NoReference, Reference, RValueReference, Pointer };
 
     inline Type() : isVolatile(false), isScoped(false), firstToken(NOTOKEN), referenceType(NoReference) {}
     inline explicit Type(const QByteArray &_name) : name(_name), isVolatile(false), isScoped(false), firstToken(NOTOKEN), referenceType(NoReference) {}
@@ -238,12 +238,16 @@ public:
     bool testFunctionAttribute(Token tok, FunctionDef *def);
 
     void checkSuperClasses(ClassDef *def);
+    void checkProperties(ClassDef* cdef);
 };
 
 inline QByteArray noRef(const QByteArray &type)
 {
-    if (type.endsWith('&'))
+    if (type.endsWith('&')) {
+        if (type.endsWith("&&"))
+            return type.left(type.length()-2);
         return type.left(type.length()-1);
+    }
     return type;
 }
 
