@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -66,8 +66,8 @@ void QHelpEngineCorePrivate::init(const QString &collectionFile,
 {
     q = helpEngineCore;
     collectionHandler = new QHelpCollectionHandler(collectionFile, helpEngineCore);
-    connect(collectionHandler, SIGNAL(error(const QString&)),
-        this, SLOT(errorReceived(const QString&)));
+    connect(collectionHandler, SIGNAL(error(QString)),
+        this, SLOT(errorReceived(QString)));
     needsSetup = true;
 }
 
@@ -119,7 +119,7 @@ bool QHelpEngineCorePrivate::setup()
         QHelpDBReader *reader = new QHelpDBReader(absFileName,
             QHelpGlobal::uniquifyConnectionName(info.fileName, this), this);
         if (!reader->init()) {
-            emit q->warning(tr("Cannot open documentation file %1: %2!")
+            emit q->warning(QHelpEngineCore::tr("Cannot open documentation file %1: %2!")
                 .arg(absFileName, reader->errorMessage()));
             continue;
         }
@@ -168,7 +168,7 @@ void QHelpEngineCorePrivate::errorReceived(const QString &msg)
     it is removed by calling removeCustomFilter(). customFilters() returns
     all defined filters.
 
-    The help engine also offers the possiblity to set and read values
+    The help engine also offers the possibility to set and read values
     in a persistant way comparable to ini files or Windows registry
     entries. For more information see setValue() or value().
 
@@ -359,19 +359,21 @@ bool QHelpEngineCore::unregisterDocumentation(const QString &namespaceName)
 */
 QString QHelpEngineCore::documentationFileName(const QString &namespaceName)
 {
-    QString res;
-    if (!d->setup())
-        return res;
-    const QHelpCollectionHandler::DocInfoList docList = d->collectionHandler->registeredDocumentations();
-    foreach(const QHelpCollectionHandler::DocInfo info, docList) {
-        if (info.namespaceName == namespaceName) {
-            QFileInfo fi(d->collectionHandler->collectionFile());
-            fi.setFile(fi.absolutePath() + QDir::separator() + info.fileName);
-            res = QDir::cleanPath(fi.absoluteFilePath());
-            break;
+    if (d->setup()) {
+        const QHelpCollectionHandler::DocInfoList docList =
+            d->collectionHandler->registeredDocumentations();
+        foreach(const QHelpCollectionHandler::DocInfo &info, docList) {
+            if (info.namespaceName == namespaceName) {
+                if (QDir::isAbsolutePath(info.fileName))
+                    return QDir::cleanPath(info.fileName);
+
+                QFileInfo fi(d->collectionHandler->collectionFile());
+                fi.setFile(fi.absolutePath() + QDir::separator() + info.fileName);
+                return QDir::cleanPath(fi.absoluteFilePath());
+            }
         }
     }
-    return res;
+    return QString();
 }
 
 /*!
@@ -384,7 +386,7 @@ QStringList QHelpEngineCore::registeredDocumentations() const
     if (!d->setup())
         return list;
     const QHelpCollectionHandler::DocInfoList docList = d->collectionHandler->registeredDocumentations();
-    foreach(const QHelpCollectionHandler::DocInfo info, docList) {
+    foreach(const QHelpCollectionHandler::DocInfo &info, docList) {
         list.append(info.namespaceName);
     }
     return list;
@@ -404,8 +406,9 @@ QStringList QHelpEngineCore::customFilters() const
 
 /*!
     Adds the new custom filter \a filterName. The filter attributes
-    are specified by \a attributes. The function returns false if
-    the filter can not be added, e.g. when the filter already exists.
+    are specified by \a attributes. If the filter already exists,
+    its attribute set is replaced. The function returns true if
+    the operation succeeded, otherwise it returns false.
 
     \sa customFilters(), removeCustomFilter()
 */
@@ -529,7 +532,7 @@ QList<QUrl> QHelpEngineCore::files(const QString namespaceName,
     url.setAuthority(namespaceName);
 
     const QStringList files = reader->files(filterAttributes, extensionFilter);
-    foreach (const QString file, files) {
+    foreach (const QString &file, files) {
         url.setPath(QLatin1String("/") + file);
         res.append(url);
     }
@@ -703,7 +706,7 @@ QVariant QHelpEngineCore::metaData(const QString &documentationFileName,
 }
 
 /*!
-    Returns a description of the last error that occured.
+    Returns a description of the last error that occurred.
 */
 QString QHelpEngineCore::error() const
 {
