@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -42,7 +42,6 @@
 /*
   generator.cpp
 */
-#include <QtCore>
 #include <qdir.h>
 #include <qdebug.h>
 #include "codemarker.h"
@@ -65,6 +64,12 @@ QMap<QString, QStringList> Generator::imgFileExts;
 QSet<QString> Generator::outputFormats;
 QStringList Generator::imageFiles;
 QStringList Generator::imageDirs;
+QStringList Generator::exampleDirs;
+QStringList Generator::exampleImgExts;
+QStringList Generator::scriptFiles;
+QStringList Generator::scriptDirs;
+QStringList Generator::styleFiles;
+QStringList Generator::styleDirs;
 QString Generator::outDir;
 QString Generator::project;
 
@@ -120,12 +125,29 @@ void Generator::initialize(const Config &config)
         if (!dirInfo.mkdir(outDir + "/images"))
             config.lastLocation().fatal(tr("Cannot create output directory '%1'")
                                         .arg(outDir + "/images"));
+        if (!dirInfo.mkdir(outDir + "/images/used-in-examples"))
+            config.lastLocation().fatal(tr("Cannot create output directory '%1'")
+                                        .arg(outDir + "/images/used-in-examples"));
+        if (!dirInfo.mkdir(outDir + "/scripts"))
+            config.lastLocation().fatal(tr("Cannot create output directory '%1'")
+                                        .arg(outDir + "/scripts"));
+        if (!dirInfo.mkdir(outDir + "/style"))
+            config.lastLocation().fatal(tr("Cannot create output directory '%1'")
+                                        .arg(outDir + "/style"));
     }
 
     imageFiles = config.getStringList(CONFIG_IMAGES);
     imageDirs = config.getStringList(CONFIG_IMAGEDIRS);
+    scriptFiles = config.getStringList(CONFIG_SCRIPTS);
+    scriptDirs = config.getStringList(CONFIG_SCRIPTDIRS);
+    styleFiles = config.getStringList(CONFIG_STYLES);
+    styleDirs = config.getStringList(CONFIG_STYLEDIRS);
+    exampleDirs = config.getStringList(CONFIG_EXAMPLEDIRS);
+    exampleImgExts = config.getStringList(CONFIG_EXAMPLES + Config::dot +
+                                          CONFIG_IMAGEEXTENSIONS);
 
-    QString imagesDotFileExtensions = CONFIG_IMAGES + Config::dot + CONFIG_FILEEXTENSIONS;
+    QString imagesDotFileExtensions =
+        CONFIG_IMAGES + Config::dot + CONFIG_FILEEXTENSIONS;
     QSet<QString> formats = config.subVars(imagesDotFileExtensions);
     QSet<QString>::ConstIterator f = formats.begin();
     while (f != formats.end()) {
@@ -138,14 +160,15 @@ void Generator::initialize(const Config &config)
     while (g != generators.end()) {
         if (outputFormats.contains((*g)->format())) {
             (*g)->initializeGenerator(config);
-            QStringList extraImages = config.getStringList(CONFIG_EXTRAIMAGES +
-                                                           Config::dot +
-                                                           (*g)->format());
+            QStringList extraImages =
+                config.getStringList(CONFIG_EXTRAIMAGES+Config::dot+(*g)->format());
             QStringList::ConstIterator e = extraImages.begin();
             while (e != extraImages.end()) {
                 QString userFriendlyFilePath;
                 QString filePath = Config::findFile(config.lastLocation(),
-                                                    imageFiles, imageDirs, *e,
+                                                    imageFiles,
+                                                    imageDirs,
+                                                    *e,
                                                     imgFileExts[(*g)->format()],
                                                     userFriendlyFilePath);
                 if (!filePath.isEmpty())
@@ -156,6 +179,47 @@ void Generator::initialize(const Config &config)
                                      "/images");
                 ++e;
             }
+
+            QStringList noExts;
+            QStringList scripts =
+                config.getStringList(CONFIG_SCRIPTS+Config::dot+(*g)->format());
+            e = scripts.begin();
+            while (e != scripts.end()) {
+                QString userFriendlyFilePath;
+                QString filePath = Config::findFile(config.lastLocation(),
+                                                    scriptFiles,
+                                                    scriptDirs,
+                                                    *e,
+                                                    noExts,
+                                                    userFriendlyFilePath);
+                if (!filePath.isEmpty())
+                    Config::copyFile(config.lastLocation(),
+                                     filePath,
+                                     userFriendlyFilePath,
+                                     (*g)->outputDir() +
+                                     "/scripts");
+                ++e;
+            }
+
+            QStringList styles =
+                config.getStringList(CONFIG_STYLES+Config::dot+(*g)->format());
+            e = styles.begin();
+            while (e != styles.end()) {
+                QString userFriendlyFilePath;
+                QString filePath = Config::findFile(config.lastLocation(),
+                                                    styleFiles,
+                                                    styleDirs,
+                                                    *e,
+                                                    noExts,
+                                                    userFriendlyFilePath);
+                if (!filePath.isEmpty())
+                    Config::copyFile(config.lastLocation(),
+                                     filePath,
+                                     userFriendlyFilePath,
+                                     (*g)->outputDir() +
+                                     "/style");
+                ++e;
+            }
         }
         ++g;
     }
@@ -217,6 +281,7 @@ void Generator::terminate()
     imageFiles.clear();
     imageDirs.clear();
     outDir = "";
+    QmlClassNode::clear();
 }
 
 Generator *Generator::generatorForFormat(const QString& format)
@@ -312,17 +377,17 @@ void Generator::generateBody(const Node *node, CodeMarker *marker)
     bool quiet = false;
 
     if (node->type() == Node::Function) {
-#if 0        
+#if 0
         const FunctionNode *func = (const FunctionNode *) node;
         if (func->isOverload() && func->metaness() != FunctionNode::Ctor)
             generateOverload(node, marker);
-#endif        
+#endif
     }
     else if (node->type() == Node::Fake) {
         const FakeNode *fake = static_cast<const FakeNode *>(node);
         if (fake->subType() == Node::Example)
             generateExampleFiles(fake, marker);
-        else if (fake->subType() == Node::File)
+        else if ((fake->subType() == Node::File) || (fake->subType() == Node::Image))
             quiet = true;
     }
 
@@ -337,7 +402,7 @@ void Generator::generateBody(const Node *node, CodeMarker *marker)
             if (func->reimplementedFrom() != 0)
                 generateReimplementedFrom(func, marker);
         }
-        
+
         if (!generateText(node->doc().body(), node, marker))
             if (node->isReimp())
                 return;
@@ -442,7 +507,7 @@ void Generator::generateBody(const Node *node, CodeMarker *marker)
             // Now we put this at the top, before the other text.
             if (func->reimplementedFrom() != 0)
                 generateReimplementedFrom(func, marker);
-#endif            
+#endif
         }
     }
 
@@ -529,59 +594,107 @@ void Generator::generateInheritedBy(const ClassNode *classe,
     }
 }
 
+/*!
+  This function is called when the documentation for an
+  example is being formatted. It outputs the list of source
+  files comprising the example, and the list of images used
+  by the example. The images are copied into a subtree of
+  \c{...doc/html/images/used-in-examples/...}
+ */
+void Generator::generateFileList(const FakeNode* fake,
+                                 CodeMarker* marker,
+                                 Node::SubType subtype,
+                                 const QString& tag)
+{
+    int count = 0;
+    Text text;
+    OpenedList openedList(OpenedList::Bullet);
+
+    text << Atom::ParaLeft << tag << Atom::ParaRight
+         << Atom(Atom::ListLeft, openedList.styleString());
+
+    foreach (const Node* child, fake->childNodes()) {
+        if (child->subType() == subtype) {
+            ++count;
+            QString file = child->name();
+            if (subtype == Node::Image) {
+                if (!file.isEmpty()) {
+                    QDir dirInfo;
+                    QString userFriendlyFilePath;
+                    QString srcPath = Config::findFile(fake->location(),
+                                                       QStringList(),
+                                                       exampleDirs,
+                                                       file,
+                                                       exampleImgExts,
+                                                       userFriendlyFilePath);
+                    userFriendlyFilePath.truncate(userFriendlyFilePath.lastIndexOf('/'));
+
+                    QString imgOutDir = outDir + "/images/used-in-examples/" + userFriendlyFilePath;
+                    if (!dirInfo.mkpath(imgOutDir))
+                        fake->location().fatal(tr("Cannot create output directory '%1'")
+                                               .arg(imgOutDir));
+
+                    QString imgOutName = Config::copyFile(fake->location(),
+                                                          srcPath,
+                                                          file,
+                                                          imgOutDir);
+                }
+
+            }
+
+            openedList.next();
+            text << Atom(Atom::ListItemNumber, openedList.numberString())
+                 << Atom(Atom::ListItemLeft, openedList.styleString())
+                 << Atom::ParaLeft
+                 << Atom(Atom::Link, file)
+                 << Atom(Atom::FormattingLeft, ATOM_FORMATTING_LINK)
+                 << file
+                 << Atom(Atom::FormattingRight, ATOM_FORMATTING_LINK)
+                 << Atom::ParaRight
+                 << Atom(Atom::ListItemRight, openedList.styleString());
+        }
+    }
+    text << Atom(Atom::ListRight, openedList.styleString());
+    if (count > 0)
+        generateText(text, fake, marker);
+}
+
 void Generator::generateExampleFiles(const FakeNode *fake, CodeMarker *marker)
 {
     if (fake->childNodes().isEmpty())
         return;
-
-    OpenedList openedList(OpenedList::Bullet);
-
-    Text text;
-    text << Atom::ParaLeft << "Files:" << Atom::ParaRight
-         << Atom(Atom::ListLeft, openedList.styleString());
-    foreach (const Node *child, fake->childNodes()) {
-        QString exampleFile = child->name();
-        openedList.next();
-        text << Atom(Atom::ListItemNumber, openedList.numberString())
-             << Atom(Atom::ListItemLeft, openedList.styleString())
-             << Atom::ParaLeft
-             << Atom(Atom::Link, exampleFile)
-             << Atom(Atom::FormattingLeft, ATOM_FORMATTING_LINK)
-             << exampleFile
-             << Atom(Atom::FormattingRight, ATOM_FORMATTING_LINK)
-             << Atom::ParaRight
-             << Atom(Atom::ListItemRight, openedList.styleString());
-    }
-    text << Atom(Atom::ListRight, openedList.styleString());
-    generateText(text, fake, marker);
+    generateFileList(fake, marker, Node::File, QString("Files:"));
+    generateFileList(fake, marker, Node::Image, QString("Images:"));
 }
 
-void Generator::generateModuleWarning(const ClassNode *classe,
-                                      CodeMarker *marker)
-{
-    QString module = classe->moduleName();
-    if (!module.isEmpty()) {
-        Text text;
-        if (!editionModuleMap["DesktopLight"].contains(module)) {
-            text << Atom::ParaLeft
-                 << Atom(Atom::FormattingLeft, ATOM_FORMATTING_BOLD)
-                 << "This class is not part of the Qt GUI Framework Edition."
-                 << Atom(Atom::FormattingRight, ATOM_FORMATTING_BOLD)
-                 << Atom::ParaRight;
+#if 0
+    QList<Generator *>::ConstIterator g = generators.begin();
+    while (g != generators.end()) {
+        if (outputFormats.contains((*g)->format())) {
+            (*g)->initializeGenerator(config);
+            QStringList extraImages =
+                config.getStringList(CONFIG_EXTRAIMAGES+Config::dot+(*g)->format());
+            QStringList::ConstIterator e = extraImages.begin();
+            while (e != extraImages.end()) {
+                QString userFriendlyFilePath;
+                QString filePath = Config::findFile(config.lastLocation(),
+                                                    imageFiles,
+                                                    imageDirs,
+                                                    *e,
+                                                    imgFileExts[(*g)->format()],
+                                                    userFriendlyFilePath);
+                if (!filePath.isEmpty())
+                    Config::copyFile(config.lastLocation(),
+                                     filePath,
+                                     userFriendlyFilePath,
+                                     (*g)->outputDir() +
+                                     "/images");
+                ++e;
+            }
         }
-        else if (module == "Qt3Support") {
-            text << Atom::ParaLeft
-                 << Atom(Atom::FormattingLeft, ATOM_FORMATTING_BOLD)
-                 << "Note to Qt GUI Framework Edition users:"
-                 << Atom(Atom::FormattingRight, ATOM_FORMATTING_BOLD)
-                 << " This class is only available in the "
-                 << Atom(Atom::AutoLink, "Qt Full Framework Edition")
-                 << "." << Atom::ParaRight;
-        }
-
-        generateText(text, classe, marker);
+        ++g;
     }
-}
+#endif
 
 QString Generator::indent(int level, const QString& markedCode)
 {
@@ -888,7 +1001,7 @@ void Generator::generateThreadSafeness(const Node *node, CodeMarker *marker)
                 }
                 ++c;
             }
-            if (!exceptions) 
+            if (!exceptions)
                 text << ".";
             else if (threadSafeness == Node::Reentrant) {
                 if (nonreentrant.isEmpty()) {
@@ -955,8 +1068,11 @@ void Generator::generateSince(const Node *node, CodeMarker *marker)
         Text text;
         text << Atom::ParaLeft
              << "This "
-             << typeString(node)
-             << " was introduced in ";
+             << typeString(node);
+        if (node->type() == Node::Enum)
+            text << " was introduced or modified in ";
+        else
+            text << " was introduced in ";
         if (project.isEmpty())
              text << "version";
         else
@@ -975,7 +1091,7 @@ void Generator::generateOverload(const Node *node, CodeMarker *marker)
     text << Atom::ParaLeft
          << "This function overloads ";
     QString t = node->name() + "()";
-    text << Atom::AutoLink << t 
+    text << Atom::AutoLink << t
          << Atom::ParaRight;
     generateText(text, node, marker);
 }
@@ -1128,6 +1244,35 @@ void Generator::appendSortedNames(Text& text,
     }
 }
 
+void Generator::appendSortedQmlNames(Text& text,
+                                     const Node* base,
+                                     const NodeList& subs,
+                                     CodeMarker *marker)
+{
+    QMap<QString,Text> classMap;
+    int index = 0;
+
+#ifdef DEBUG_MULTIPLE_QDOCCONF_FILES
+    qDebug() << "Generator::appendSortedQmlNames():" << base->name() << "is inherited by...";
+#endif
+    for (int i = 0; i < subs.size(); ++i) {
+        Text t;
+#ifdef DEBUG_MULTIPLE_QDOCCONF_FILES
+        qDebug() << "    " << subs[i]->name();
+#endif
+        appendFullName(t, subs[i], base, marker);
+        classMap[t.toString().toLower()] = t;
+    }
+
+    QStringList names = classMap.keys();
+    names.sort();
+
+    foreach (const QString &name, names) {
+        text << classMap[name];
+        text << separator(index++, names.count());
+    }
+}
+
 int Generator::skipAtoms(const Atom *atom, Atom::Type type) const
 {
     int skipAhead = 0;
